//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <IniFiles.hpp>
#include "App_Cards.h"
#include "App_Data.h"
#include "Moa_AutoPointers.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_cards {
//---------------------------------------------------------------------------
__fastcall TFlashAbout::TFlashAbout() :
	TObject(),
	FTitle(),
	FDescription()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TFlashAbout::~TFlashAbout()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TFlashAbout::Assign(TFlashAbout* ASource)
{
	if(ASource == this) {
		return;
	}
	FTitle = ASource->FTitle;
	FDescription = ASource->FDescription;
}
//---------------------------------------------------------------------------
void __fastcall TFlashAbout::ReadData(TFlashEntries* AEntries)
{
	TAutoPointer<TFlashAbout> LAbout = new TFlashAbout();
	{
		TAutoPointer<TFlashEntry> LEntry = AEntries->Read("About");
		LAbout->Title = LEntry->Read("Title", LAbout->Title);
		LAbout->Description = LEntry->Read("Description", LAbout->Description);
	}
	Assign(LAbout);
}
//---------------------------------------------------------------------------
void __fastcall TFlashAbout::WriteData(TFlashEntries* AEntries)
{
	TAutoPointer<TFlashEntry> LEntry = new TFlashEntry();
	LEntry->Write("Title", Title);
	LEntry->Write("Description", Description);
	AEntries->Write("About", LEntry);
	LEntry.Release();
}
//---------------------------------------------------------------------------
bool __fastcall TFlashAbout::HasTitle()
{
	return Title.Length() > 0;
}
//---------------------------------------------------------------------------
bool __fastcall TFlashAbout::HasDescription()
{
	return Description.Length() > 0;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TFlashPage::TFlashPage() :
	TObject(),
	FText(),
	FFile()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TFlashPage::~TFlashPage()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TFlashPage::Assign(TFlashPage* ASource)
{
	if(ASource == this) {
		return;
	}
	FText = ASource->FText;
	FFile = ASource->FFile;
}
//---------------------------------------------------------------------------
bool __fastcall TFlashPage::HasFile()
{
	return File.Length() > 0;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TFlashCard::Initialize()
{
	try {
		FQuestion = new TFlashPage();
		FAnswer = new TFlashPage();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TFlashCard::Uninitialize()
{
	delete FAnswer;
	FAnswer = NULL;

	delete FQuestion;
	FQuestion = NULL;
}
//---------------------------------------------------------------------------
__fastcall TFlashCard::TFlashCard() :
	TObject(),
	FQuestion(NULL),
	FAnswer(NULL),
	FHint(),
	FNote()
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TFlashCard::~TFlashCard()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TFlashCard::Assign(TFlashCard* ASource)
{
	if(ASource == this) {
		return;
	}
	FQuestion->Assign(ASource->FQuestion);
	FAnswer->Assign(ASource->FAnswer);
	FHint = ASource->FHint;
	FNote = ASource->FNote;
}
//---------------------------------------------------------------------------
void __fastcall TFlashCard::ReadData(TFlashEntries* AEntries)
{
	TAutoPointer<TFlashCard> LCard = new TFlashCard();
	{
		TAutoPointer<TFlashEntry> LEntry = AEntries->Read("Card");
		LCard->Question->Text = LEntry->Read("Question.Text", LCard->Question->Text);
		LCard->Question->File = LEntry->Read("Question.File", LCard->Question->File);
		LCard->Answer->Text = LEntry->Read("Answer.Text", LCard->Answer->Text);
		LCard->Answer->File = LEntry->Read("Answer.File", LCard->Answer->File);
		LCard->Hint = LEntry->Read("Hint", LCard->Hint);
		LCard->Note = LEntry->Read("Note", LCard->Note);
	}
	Assign(LCard);
}
//---------------------------------------------------------------------------
void __fastcall TFlashCard::WriteData(TFlashEntries* AEntries)
{
	TAutoPointer<TFlashEntry> LEntry = new TFlashEntry();
	LEntry->Write("Question.Text", Question->Text);
	LEntry->Write("Question.File", Question->File);
	LEntry->Write("Answer.Text", Answer->Text);
	LEntry->Write("Answer.File", Answer->File);
	LEntry->Write("Hint", Hint);
	LEntry->Write("Note", Note);
	AEntries->Write("Card", LEntry);
	LEntry.Release();
}
//---------------------------------------------------------------------------
bool __fastcall TFlashCard::HasHint()
{
	return Hint.Length() > 0;
}
//---------------------------------------------------------------------------
bool __fastcall TFlashCard::HasNote()
{
	return Note.Length() > 0;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TFlashCards::Initialize()
{
	try {
		FList = new TTypedObjectList<TFlashCard>();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TFlashCards::Uninitialize()
{
	delete FList;
	FList = NULL;
}
//---------------------------------------------------------------------------
TFlashCard* __fastcall TFlashCards::GetItem(int AIndex)
{
	return List->Items[AIndex];
}
//---------------------------------------------------------------------------
int __fastcall TFlashCards::GetCount()
{
	return List->Count;
}
//---------------------------------------------------------------------------
__fastcall TFlashCards::TFlashCards() :
	TObject(),
	FList(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TFlashCards::~TFlashCards()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
int __fastcall TFlashCards::Find(TFlashCard* AItem)
{
	return List->IndexOf(AItem);
}
//---------------------------------------------------------------------------
void __fastcall TFlashCards::ReadData(TFlashEntries* AEntries)
{
	TAutoPointer<TObjectList> LCards = new TObjectList();
	while(AEntries->Contains("Card")) {
		TAutoPointer<TFlashCard> LCard = new TFlashCard();
		LCard->ReadData(AEntries);
		LCards->Add(LCard);
		LCard.Release();
	}
	List->Assign(LCards);
	LCards->OwnsObjects = false;
}
//---------------------------------------------------------------------------
void __fastcall TFlashCards::WriteData(TFlashEntries* AEntries)
{
	for(int LI = 0; LI < Count; ++LI) {
		Items[LI]->WriteData(AEntries);
	}
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TFlashDeck::Initialize()
{
	try {
		FAbout = new TFlashAbout();
		FCards = new TFlashCards();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TFlashDeck::Uninitialize()
{
	delete FCards;
	FCards = NULL;

	delete FAbout;
	FAbout = NULL;
}
//---------------------------------------------------------------------------
__fastcall TFlashDeck::TFlashDeck() :
	TObject(),
	FAbout(NULL),
	FCards(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TFlashDeck::~TFlashDeck()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TFlashDeck::ReadData(TFlashEntries* AEntries)
{
	TAutoPointer<TFlashAbout> LAbout = new TFlashAbout();
	LAbout->ReadData(AEntries);
	TAutoPointer<TFlashCards> LCards = new TFlashCards();
	LCards->ReadData(AEntries);
	FAbout = LAbout.Reset(FAbout);
	FCards = LCards.Reset(FCards);
}
//---------------------------------------------------------------------------
void __fastcall TFlashDeck::WriteData(TFlashEntries* AEntries)
{
	About->WriteData(AEntries);
	Cards->WriteData(AEntries);
}
//---------------------------------------------------------------------------
void __fastcall TFlashDeck::LoadFromFile(const AnsiString& AFile)
{
	TAutoPointer<TFlashEntries> LEntries = new TFlashEntries();
	LEntries->LoadFromFile(AFile);
	ReadData(LEntries);
}
//---------------------------------------------------------------------------
void __fastcall TFlashDeck::SaveToFile(const AnsiString& AFile)
{
	TAutoPointer<TFlashEntries> LEntries = new TFlashEntries();
	WriteData(LEntries);
	LEntries->SaveToFile(AFile);
}
//---------------------------------------------------------------------------
} // End namespace App_cards
//---------------------------------------------------------------------------

