//---------------------------------------------------------------------------
#ifndef App_CardsH
#define App_CardsH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include "Moa_TypedLists.h"
namespace App_data { class TFlashEntries; } using namespace App_data;
//---------------------------------------------------------------------------
namespace App_cards {
//---------------------------------------------------------------------------
class ECardError : public Exception
{
public:
	__fastcall ECardError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class TFlashAbout : public TObject
{
private:
	AnsiString FTitle;
	AnsiString FDescription;
protected:
public:
	__fastcall TFlashAbout();
	virtual __fastcall ~TFlashAbout();
	virtual void __fastcall Assign(TFlashAbout* ASource);
	virtual void __fastcall ReadData(TFlashEntries* AEntries);
	virtual void __fastcall WriteData(TFlashEntries* AEntries);
	virtual bool __fastcall HasTitle();
	virtual bool __fastcall HasDescription();
	__property AnsiString Title = {read=FTitle, write=FTitle};
	__property AnsiString Description = {read=FDescription, write=FDescription};
};
//---------------------------------------------------------------------------
class TFlashPage : public TObject
{
private:
	AnsiString FText;
	AnsiString FFile;
protected:
public:
	__fastcall TFlashPage();
	virtual __fastcall ~TFlashPage();
	virtual void __fastcall Assign(TFlashPage* ASource);
	virtual bool __fastcall HasFile();
	__property AnsiString Text = {read=FText, write=FText};
	__property AnsiString File = {read=FFile, write=FFile};
};
//---------------------------------------------------------------------------
class TFlashCard : public TObject
{
private:
	TFlashPage* FQuestion;
	TFlashPage* FAnswer;
	AnsiString FHint;
	AnsiString FNote;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
protected:
public:
	__fastcall TFlashCard();
	virtual __fastcall ~TFlashCard();
	virtual void __fastcall Assign(TFlashCard* ASource);
	virtual void __fastcall ReadData(TFlashEntries* AEntries);
	virtual void __fastcall WriteData(TFlashEntries* AEntries);
	virtual bool __fastcall HasHint();
	virtual bool __fastcall HasNote();
	__property TFlashPage* Question = {read=FQuestion};
	__property TFlashPage* Answer = {read=FAnswer};
	__property AnsiString Hint = {read=FHint, write=FHint};
	__property AnsiString Note = {read=FNote, write=FNote};
};
//---------------------------------------------------------------------------
class TFlashCards : public TObject
{
private:
	TTypedObjectList<TFlashCard>* FList;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	TFlashCard* __fastcall GetItem(int AIndex);
	int __fastcall GetCount();
	__property TTypedObjectList<TFlashCard>* List = {read=FList};
protected:
public:
	__fastcall TFlashCards();
	virtual __fastcall ~TFlashCards();
	virtual int __fastcall Find(TFlashCard* AItem);
	virtual void __fastcall ReadData(TFlashEntries* AEntries);
	virtual void __fastcall WriteData(TFlashEntries* AEntries);
	__property TFlashCard* Items[int AIndex] = {read=GetItem};
	__property int Count = {read=GetCount};
};
//---------------------------------------------------------------------------
class TFlashDeck : public TObject
{
private:
	TFlashAbout* FAbout;
	TFlashCards* FCards;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
protected:
public:
	__fastcall TFlashDeck();
	virtual __fastcall ~TFlashDeck();
	virtual void __fastcall ReadData(TFlashEntries* AEntries);
	virtual void __fastcall WriteData(TFlashEntries* AEntries);
	virtual void __fastcall LoadFromFile(const AnsiString& AFile);
	virtual void __fastcall SaveToFile(const AnsiString& AFile);
	__property TFlashAbout* About = {read=FAbout};
	__property TFlashCards* Cards = {read=FCards};
};
//---------------------------------------------------------------------------
} // End namespace App_cards
using namespace App_cards;
//---------------------------------------------------------------------------
#endif

