//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <IniFiles.hpp>
#include "App_Data.h"
#include "Moa_AutoPointers.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_data {
//---------------------------------------------------------------------------
void __fastcall TFlashEntry::Initialize()
{
	try {
		FList = new TStringList();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TFlashEntry::Uninitialize()
{
	delete FList;
	FList = NULL;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TFlashEntry::GetAsText()
{
	return List->Text;
}
//---------------------------------------------------------------------------
void __fastcall TFlashEntry::SetAsText(const AnsiString& AAsText)
{
	List->Text = AAsText;
}
//---------------------------------------------------------------------------
__fastcall TFlashEntry::TFlashEntry() :
	TObject(),
	FList(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TFlashEntry::~TFlashEntry()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
AnsiString __fastcall TFlashEntry::Read(const AnsiString& AKey, const AnsiString& ADefaultValue)
{
	if(Contains(AKey)) {
		return List->Values[AKey];
	}
	else {
		return ADefaultValue;
	}
}
//---------------------------------------------------------------------------
void __fastcall TFlashEntry::Write(const AnsiString& AKey, const AnsiString& AValue)
{
	List->Values[AKey] = AValue;
}
//---------------------------------------------------------------------------
bool __fastcall TFlashEntry::Contains(const AnsiString& AKey)
{
	return List->IndexOfName(AKey) >= 0;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TFlashEntries::FreeObjects(TStrings* AStrings)
{
	for(int LI = 0; LI < AStrings->Count; ++LI) {
		delete AStrings->Objects[LI];
		AStrings->Objects[LI] = NULL;
	}
}
//---------------------------------------------------------------------------
void __fastcall TFlashEntries::Initialize()
{
	try {
		FList = new TStringList();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TFlashEntries::Uninitialize()
{
	delete FList;
	FList = NULL;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TFlashEntries::GetAsText()
{
	TAutoPointer<TStrings> LStrings = new TStringList();
	for(int LI = 0; LI < List->Count; ++LI) {
		LStrings->Add("[" + List->Strings[LI] + "]");
		LStrings->Add(static_cast<TFlashEntry*>(List->Objects[LI])->AsText);
	}
	return LStrings->Text;
}
//---------------------------------------------------------------------------
void __fastcall TFlashEntries::SetAsText(const AnsiString& AAsText)
{
	TAutoPointer<TStrings> LList = new TStringList();
	try {
		{
			TAutoPointer<TMemIniFile> LParser = new TMemIniFile(AnsiString());
			{
				TAutoPointer<TStrings> LStrings = new TStringList();
				LStrings->Text = AAsText;
				LParser->SetStrings(LStrings);
			}
			TAutoPointer<TStrings> LNames = new TStringList();
			LParser->ReadSections(LNames);
			for(int LI = 0; LI < LNames->Count; ++LI) {
				int LIndex = LNames->Count - 1 - LI;
				TAutoPointer<TFlashEntry> LEntry = new TFlashEntry();
				{
					TAutoPointer<TStrings> LStrings = new TStringList();
					LParser->ReadSectionValues(LNames->Strings[LIndex], LStrings);
					LParser->EraseSection(LNames->Strings[LIndex]);
					LEntry->AsText = LStrings->Text;
				}
				LList->InsertObject(0, LNames->Strings[LIndex], LEntry);
				LEntry.Release();
			}
		}
		FreeObjects(List);
		List->Assign(LList);
	}
	catch(...) {
		FreeObjects(LList);
		throw;
	}
}
//---------------------------------------------------------------------------
__fastcall TFlashEntries::TFlashEntries() :
	TObject(),
	FList(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TFlashEntries::~TFlashEntries()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
TFlashEntry* __fastcall TFlashEntries::Read(const AnsiString& AName)
{
	TAutoPointer<TFlashEntry> LItem;
	int LIndex = List->IndexOf(AName);
	if(LIndex < 0) {
		LItem = new TFlashEntry();
	}
	else {
		LItem = static_cast<TFlashEntry*>(List->Objects[LIndex]);
		List->Delete(LIndex);
	}
	return LItem.Release();
}
//---------------------------------------------------------------------------
void __fastcall TFlashEntries::Write(const AnsiString& AName, TFlashEntry* AEntry)
{
	List->AddObject(AName, AEntry);
}
//---------------------------------------------------------------------------
bool __fastcall TFlashEntries::Contains(const AnsiString& AName)
{
	return List->IndexOf(AName) >= 0;
}
//---------------------------------------------------------------------------
void __fastcall TFlashEntries::LoadFromFile(const AnsiString& AFile)
{
	try {
		TAutoPointer<TStrings> LStrings = new TStringList();
		LStrings->LoadFromFile(AFile);
		AsText = LStrings->Text;
	}
	catch(EStreamError& LException) {
		LException.Message = "The program could not load the data.";
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TFlashEntries::SaveToFile(const AnsiString& AFile)
{
	try {
		TAutoPointer<TStrings> LStrings = new TStringList();
		LStrings->Text = AsText;
		LStrings->SaveToFile(AFile);
	}
	catch(EStreamError& LException) {
		LException.Message = "The program could not save the data.";
		throw;
	}
}
//---------------------------------------------------------------------------
} // End namespace App_data
//---------------------------------------------------------------------------

