//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <Math.hpp>
#include "App_Cards.h"
#include "App_Exams.h"
#include "Moa_AutoPointers.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_exams {
//---------------------------------------------------------------------------
TFlashPage* __fastcall TExamCard::GetQuestion()
{
	if(Reverse) {
		return Source->Answer;
	}
	return Source->Question;
}
//---------------------------------------------------------------------------
TFlashPage* __fastcall TExamCard::GetAnswer()
{
	if(Reverse) {
		return Source->Question;
	}
	return Source->Answer;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TExamCard::GetHint()
{
	return Source->Hint;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TExamCard::GetNote()
{
	return Source->Note;
}
//---------------------------------------------------------------------------
__fastcall TExamCard::TExamCard(TFlashCard* ASource) :
	TObject(),
	FSource(ASource),
	FReverse(false)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TExamCard::~TExamCard()
{
	// Do nothing
}
//---------------------------------------------------------------------------
bool __fastcall TExamCard::HasHint()
{
	return Source->HasHint();
}
//---------------------------------------------------------------------------
bool __fastcall TExamCard::HasNote()
{
	return Source->HasNote();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TExamCards::Initialize()
{
	try {
		FList = new TTypedObjectList<TExamCard>();
		FList->Capacity = Source->Count;
		for(int LI = 0; LI < Source->Count; ++LI) {
			TAutoPointer<TExamCard> LCard = new TExamCard(Source->Items[LI]);
			FList->Add(LCard);
			LCard.Release();
		}
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TExamCards::Uninitialize()
{
	delete FList;
	FList = NULL;
}
//---------------------------------------------------------------------------
TExamCard* __fastcall TExamCards::GetItem(int AIndex)
{
	return List->Items[AIndex];
}
//---------------------------------------------------------------------------
int __fastcall TExamCards::GetCount()
{
	return List->Count;
}
//---------------------------------------------------------------------------
__fastcall TExamCards::TExamCards(TFlashCards* ASource) :
	TObject(),
	FSource(ASource),
	FList(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TExamCards::~TExamCards()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TExamCards::Shuffle()
{
	for(int LI = 0; LI < List->Count; ++LI) {
		List->Exchange(LI, RandomRange(0, List->Count - 1));
	}
}
//---------------------------------------------------------------------------
int __fastcall TExamCards::Find(TExamCard* AItem)
{
	return List->IndexOf(AItem);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TExamCardNavigator::Initialize(TExamCards* ACards)
{
	try {
		FList = new TTypedObjectList<TExamCard>(false);
		FList->Capacity = ACards->Count;
		for(int LI = 0; LI < ACards->Count; ++LI) {
			FList->Add(ACards->Items[LI]);
		}
		FHistory = new TTypedObjectList<TExamCard>(false);
		FHistory->Capacity = ACards->Count;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TExamCardNavigator::Uninitialize()
{
	delete FHistory;
	FHistory = NULL;

	delete FList;
	FList = NULL;
}
//---------------------------------------------------------------------------
TExamCard* __fastcall TExamCardNavigator::GetCurrentCard()
{
	if(!IsStarted() || IsEnded() || Current == All) {
		throw EExamError("The pogram cannot get the card.");
	}
	return List->Items[Position];
}
//---------------------------------------------------------------------------
int __fastcall TExamCardNavigator::GetCurrent()
{
	return Position;
}
//---------------------------------------------------------------------------
int __fastcall TExamCardNavigator::GetAll()
{
	return List->Count;
}
//---------------------------------------------------------------------------
TExamCardNavigator::TStates __fastcall TExamCardNavigator::GetStates()
{
	return FStates;
}
//---------------------------------------------------------------------------
void __fastcall TExamCardNavigator::DoStart()
{
	if(OnStart != NULL) {
		OnStart(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TExamCardNavigator::DoEnd()
{
	if(OnEnd != NULL) {
		OnEnd(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TExamCardNavigator::DoProgress()
{
	if(OnProgress != NULL) {
		OnProgress(this);
	}
}
//---------------------------------------------------------------------------
__fastcall TExamCardNavigator::TExamCardNavigator(TExamCards* ACards) :
	TObject(),
	FList(NULL),
	FHistory(NULL),
	FPosition(0),
	FStates(),
	FOnStart(NULL),
	FOnEnd(NULL),
	FOnProgress(NULL)
{
	Initialize(ACards);
}
//---------------------------------------------------------------------------
__fastcall TExamCardNavigator::~TExamCardNavigator()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TExamCardNavigator::Start()
{
	if(!CanStart()) {
		throw EExamError("The pogram cannot start the exam.");
	}
	FStates << stStarted;
	DoStart();
	DoProgress();
}
//---------------------------------------------------------------------------
bool __fastcall TExamCardNavigator::CanStart()
{
	return !IsStarted() && !IsEnded();
}
//---------------------------------------------------------------------------
bool __fastcall TExamCardNavigator::IsStarted()
{
	return States.Contains(stStarted);
}
//---------------------------------------------------------------------------
void __fastcall TExamCardNavigator::End()
{
	if(!CanEnd()) {
		throw EExamError("The pogram cannot end the exam.");
	}
	FStates << stEnded;
	DoEnd();
}
//---------------------------------------------------------------------------
bool __fastcall TExamCardNavigator::CanEnd()
{
	return IsStarted() && !IsEnded();
}
//---------------------------------------------------------------------------
bool __fastcall TExamCardNavigator::IsEnded()
{
	return States.Contains(stEnded);
}
//---------------------------------------------------------------------------
void __fastcall TExamCardNavigator::Retry(int AInterval)
{
	if(!CanRetry() || AInterval < 0) {
		throw EExamError("The pogram cannot retry the card.");
	}
	History->Add(CurrentCard);
	List->Move(Position, Position + Math::Min(AInterval, All - Current - 1));
	DoProgress();
}
//---------------------------------------------------------------------------
bool __fastcall TExamCardNavigator::CanRetry()
{
	return IsStarted() && !IsEnded() && Current < All;
}
//---------------------------------------------------------------------------
void __fastcall TExamCardNavigator::GoBack()
{
	if(!CanGoBack()) {
		throw EExamError("The pogram cannot move to the previous card.");
	}
	if(Position > 0 && List->Items[Position - 1] == History->Last()) {
		--FPosition;
	}
	else {
		List->Move(List->IndexOf(History->Last()), Position);
	}
	History->Delete(History->Count - 1);
	DoProgress();
}
//---------------------------------------------------------------------------
bool __fastcall TExamCardNavigator::CanGoBack()
{
	return IsStarted() && !IsEnded() && History->Count > 0;
}
//---------------------------------------------------------------------------
void __fastcall TExamCardNavigator::GoForward()
{
	if(!CanGoForward()) {
		throw EExamError("The pogram cannot move to the next card.");
	}
	History->Add(CurrentCard);
	++FPosition;
	DoProgress();
}
//---------------------------------------------------------------------------
bool __fastcall TExamCardNavigator::CanGoForward()
{
	return IsStarted() && !IsEnded() && Current < All;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
TExamStepNavigator::TSteps __fastcall TExamStepNavigator::GetSteps()
{
	return FSteps;
}
//---------------------------------------------------------------------------
void __fastcall TExamStepNavigator::DoShowQuestion()
{
	if(OnShowQuestion) {
		OnShowQuestion(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TExamStepNavigator::DoShowHint()
{
	if(OnShowHint) {
		OnShowHint(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TExamStepNavigator::DoShowAnswer()
{
	if(OnShowAnswer) {
		OnShowAnswer(this);
	}
}
//---------------------------------------------------------------------------
__fastcall TExamStepNavigator::TExamStepNavigator() :
	TObject(),
	FSteps(),
	FOnShowQuestion(NULL),
	FOnShowHint(NULL),
	FOnShowAnswer(NULL)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TExamStepNavigator::~TExamStepNavigator()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TExamStepNavigator::Proceed()
{
	if(!CanProceed()) {
		throw EExamError("The program cannot move to the next step.");
	}
	if(IsQuestionShown()) {
		ShowAnswer();
	}
	else {
		ShowQuestion();
	}
}
//---------------------------------------------------------------------------
bool __fastcall TExamStepNavigator::CanProceed()
{
	return !IsAnswerShown();
}
//---------------------------------------------------------------------------
void __fastcall TExamStepNavigator::ShowQuestion()
{
	if(!CanShowQuestion()) {
		throw EExamError("The program cannot show the question.");
	}
	FSteps << stQuestion;
	DoShowQuestion();
}
//---------------------------------------------------------------------------
bool __fastcall TExamStepNavigator::CanShowQuestion()
{
	return !IsQuestionShown();
}
//---------------------------------------------------------------------------
bool __fastcall TExamStepNavigator::IsQuestionShown()
{
	return Steps.Contains(stQuestion);
}
//---------------------------------------------------------------------------
void __fastcall TExamStepNavigator::ShowHint()
{
	if(!CanShowHint()) {
		throw EExamError("The program cannot show the hint.");
	}
	FSteps << stHint;
	DoShowHint();
}
//---------------------------------------------------------------------------
bool __fastcall TExamStepNavigator::CanShowHint()
{
	return IsQuestionShown() && !IsHintShown() && !IsAnswerShown();
}
//---------------------------------------------------------------------------
bool __fastcall TExamStepNavigator::IsHintShown()
{
	return Steps.Contains(stHint);
}
//---------------------------------------------------------------------------
void __fastcall TExamStepNavigator::ShowAnswer()
{
	if(!CanShowAnswer()) {
		throw EExamError("The program cannot show the answer.");
	}
	FSteps << stAnswer;
	DoShowAnswer();
}
//---------------------------------------------------------------------------
bool __fastcall TExamStepNavigator::CanShowAnswer()
{
	return IsQuestionShown() && !IsAnswerShown();
}
//---------------------------------------------------------------------------
bool __fastcall TExamStepNavigator::IsAnswerShown()
{
	return Steps.Contains(stAnswer);
}
//---------------------------------------------------------------------------
void __fastcall TExamStepNavigator::Reset()
{
	FSteps.Clear();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TExamTimekeeper::Initialize()
{
	try {
		FTimer = new TTimer(NULL);
		FTimer->Enabled = false;
		FTimer->Interval = 3000;
		FTimer->OnTimer = TimerTimer;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TExamTimekeeper::Uninitialize()
{
	delete FTimer;
	FTimer = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TExamTimekeeper::TimerTimer(TObject* ASender)
{
	Timer->Enabled = false;
	FTimeUp = true;
	DoTimeUp();
}
//---------------------------------------------------------------------------
int __fastcall TExamTimekeeper::GetTimeLimit()
{
	return Timer->Interval;
}
//---------------------------------------------------------------------------
void __fastcall TExamTimekeeper::SetTimeLimit(int ATimeLimit)
{
	Timer->Interval = ATimeLimit;
}
//---------------------------------------------------------------------------
void __fastcall TExamTimekeeper::DoTimeUp()
{
	if(OnTimeUp != NULL) {
		OnTimeUp(this);
	}
}
//---------------------------------------------------------------------------
__fastcall TExamTimekeeper::TExamTimekeeper() :
	TObject(),
	FTimer(NULL),
	FTimeUp(false),
	FOnTimeUp(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TExamTimekeeper::~TExamTimekeeper()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TExamTimekeeper::Start()
{
	if(Timer->Enabled) {
		throw EExamError("The program cannot start the timekeeper.");
	}
	Reset();
	if(Timer->Interval == 0) {
		Timer->OnTimer(Timer);
	}
	Timer->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TExamTimekeeper::Stop()
{
	Timer->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TExamTimekeeper::Reset()
{
	Stop();
	FTimeUp = false;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
bool TExamRules::operator ==(const TExamRules& AOther) const
{
	if(&AOther == this) {
		return true;
	}
	return
		Shuffle == AOther.Shuffle &&
		Reverse == AOther.Reverse &&
		AutoHint == AOther.AutoHint &&
		AutoAnswer == AOther.AutoAnswer &&
		LimitTime == AOther.LimitTime &&
		TimeLimit == AOther.TimeLimit &&
		ChargeRetry == AOther.ChargeRetry &&
		RetryInterval == AOther.RetryInterval;
}
//---------------------------------------------------------------------------
bool TExamRules::operator !=(const TExamRules& AOther) const
{
	return !(*this == AOther);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TExam::Initialize(TFlashCards* ACards)
{
	try {
		FCards = new TExamCards(ACards);
		if(Rules.Shuffle) {
			FCards->Shuffle();
		}
		for(int LI = 0; LI < FCards->Count; ++LI) {
			FCards->Items[LI]->Reverse = Rules.Reverse;
		}
		FCardNavigator = new TExamCardNavigator(FCards);
		FCardNavigator->OnStart = CardNavigatorStart;
		FCardNavigator->OnEnd = CardNavigatorEnd;
		FCardNavigator->OnProgress = CardNavigatorProgress;
		FStepNavigator = new TExamStepNavigator();
		FStepNavigator->OnShowQuestion = StepNavigatorShowQuestion;
		FStepNavigator->OnShowHint = StepNavigatorShowHint;
		FStepNavigator->OnShowAnswer = StepNavigatorShowAnswer;
		if(Rules.LimitTime) {
			FTimekeeper = new TExamTimekeeper();
			FTimekeeper->TimeLimit = Rules.TimeLimit;
			FTimekeeper->OnTimeUp = TimekeeperTimeUp;
		}
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TExam::Uninitialize()
{
	delete FTimekeeper;
	FTimekeeper = NULL;

	delete FStepNavigator;
	FStepNavigator = NULL;

	delete FCardNavigator;
	FCardNavigator = NULL;

	delete FCards;
	FCards = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TExam::CardNavigatorStart(TObject* ASender)
{
	DoStart();
}
//---------------------------------------------------------------------------
void __fastcall TExam::CardNavigatorEnd(TObject* ASender)
{
	if(Rules.LimitTime) {
		Timekeeper->Stop();
	}
	DoEnd();
}
//---------------------------------------------------------------------------
void __fastcall TExam::CardNavigatorProgress(TObject* ASender)
{
	if(Rules.LimitTime) {
		Timekeeper->Stop();
	}
	DoProgress();
	StepNavigator->Reset();
	if(Current < All) {
		Proceed();
	}
}
//---------------------------------------------------------------------------
void __fastcall TExam::StepNavigatorShowQuestion(TObject* ASender)
{
	DoShowQuestion();
	if(Rules.LimitTime) {
		Timekeeper->Start();
	}
}
//---------------------------------------------------------------------------
void __fastcall TExam::StepNavigatorShowHint(TObject* ASender)
{
	DoShowHint();
}
//---------------------------------------------------------------------------
void __fastcall TExam::StepNavigatorShowAnswer(TObject* ASender)
{
	if(Rules.LimitTime) {
		Timekeeper->Stop();
	}
	DoShowAnswer();
}
//---------------------------------------------------------------------------
void __fastcall TExam::TimekeeperTimeUp(TObject* ASender)
{
	DoTimeUp();
}
//---------------------------------------------------------------------------
TExamCard* __fastcall TExam::GetCurrentCard()
{
	return CardNavigator->CurrentCard;
}
//---------------------------------------------------------------------------
int __fastcall TExam::GetCurrent()
{
	return CardNavigator->Current;
}
//---------------------------------------------------------------------------
int __fastcall TExam::GetAll()
{
	return CardNavigator->All;
}
//---------------------------------------------------------------------------
void __fastcall TExam::DoStart()
{
	if(OnStart != NULL) {
		OnStart(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TExam::DoEnd()
{
	if(OnEnd != NULL) {
		OnEnd(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TExam::DoProgress()
{
	if(OnProgress != NULL) {
		OnProgress(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TExam::DoShowQuestion()
{
	if(OnShowQuestion != NULL) {
		OnShowQuestion(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TExam::DoShowHint()
{
	if(OnShowHint != NULL) {
		OnShowHint(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TExam::DoShowAnswer()
{
	if(OnShowAnswer != NULL) {
		OnShowAnswer(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TExam::DoTimeUp()
{
	if(OnTimeUp != NULL) {
		OnTimeUp(this);
	}
}
//---------------------------------------------------------------------------
__fastcall TExam::TExam(TFlashCards* ACards, const TExamRules& ARules) :
	TObject(),
	FCards(NULL),
	FCardNavigator(NULL),
	FStepNavigator(NULL),
	FTimekeeper(NULL),
	FRules(ARules),
	FOnStart(NULL),
	FOnEnd(NULL),
	FOnProgress(NULL),
	FOnShowQuestion(NULL),
	FOnShowHint(NULL),
	FOnShowAnswer(NULL),
	FOnTimeUp(NULL)
{
	Initialize(ACards);
}
//---------------------------------------------------------------------------
__fastcall TExam::~TExam()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TExam::Start()
{
	if(!CanStart()) {
		throw EExamError("The program cannot start the exam.");
	}
	CardNavigator->Start();
}
//---------------------------------------------------------------------------
bool __fastcall TExam::CanStart()
{
	return CardNavigator->CanStart();
}
//---------------------------------------------------------------------------
bool __fastcall TExam::IsStarted()
{
	return CardNavigator->IsStarted();
}
//---------------------------------------------------------------------------
bool __fastcall TExam::IsEnded()
{
	return CardNavigator->IsEnded();
}
//---------------------------------------------------------------------------
void __fastcall TExam::Stop()
{
	if(!CanStop()) {
		throw EExamError("The program cannot stop the exam.");
	}
	CardNavigator->End();
}
//---------------------------------------------------------------------------
bool __fastcall TExam::CanStop()
{
	return CardNavigator->CanEnd();
}
//---------------------------------------------------------------------------
void __fastcall TExam::Proceed()
{
	if(!CanProceed()) {
		throw EExamError("The program cannot move to the next step.");
	}
	if(Current == All) {
		CardNavigator->End();
	}
	else {
		if(StepNavigator->CanProceed()) {
			StepNavigator->Proceed();
			if(Rules.AutoHint && CanShowHint()) {
				ShowHint();
			}
			if(Rules.AutoAnswer && CanShowAnswer()) {
				ShowAnswer();
			}
		}
		else {
			if(Rules.LimitTime && Rules.ChargeRetry && Timekeeper->TimeUp) {
				Retry();
			}
			else {
				GoForward();
			}
		}
	}
}
//---------------------------------------------------------------------------
bool __fastcall TExam::CanProceed()
{
	return IsStarted() && !IsEnded();
}
//---------------------------------------------------------------------------
void __fastcall TExam::Retry()
{
	if(!CanRetry()) {
		throw EExamError("The program cannot retry the card.");
	}
	CardNavigator->Retry(Rules.RetryInterval);
}
//---------------------------------------------------------------------------
bool __fastcall TExam::CanRetry()
{
	return Current < All;
}
//---------------------------------------------------------------------------
void __fastcall TExam::ShowHint()
{
	if(!CanShowHint()) {
		throw EExamError("The program cannot show the hint.");
	}
	StepNavigator->ShowHint();
}
//---------------------------------------------------------------------------
bool __fastcall TExam::CanShowHint()
{
	return Current < All && StepNavigator->CanShowHint() && CurrentCard->HasHint();
}
//---------------------------------------------------------------------------
void __fastcall TExam::ShowAnswer()
{
	if(!CanShowAnswer()) {
		throw EExamError("The program cannot show the answer.");
	}
	StepNavigator->ShowAnswer();
}
//---------------------------------------------------------------------------
bool __fastcall TExam::CanShowAnswer()
{
	return Current < All && StepNavigator->CanShowAnswer();
}
//---------------------------------------------------------------------------
void __fastcall TExam::GoBack()
{
	if(!CanGoBack()) {
		throw EExamError("The program cannot move to the previous card.");
	}
	CardNavigator->GoBack();
}
//---------------------------------------------------------------------------
bool __fastcall TExam::CanGoBack()
{
	return CardNavigator->CanGoBack();
}
//---------------------------------------------------------------------------
void __fastcall TExam::GoForward()
{
	if(!CanGoForward()) {
		throw EExamError("The program cannot move to the next card.");
	}
	CardNavigator->GoForward();
}
//---------------------------------------------------------------------------
bool __fastcall TExam::CanGoForward()
{
	return CardNavigator->CanGoForward();
}
//---------------------------------------------------------------------------
} // End namespace App_exams
//---------------------------------------------------------------------------

