//---------------------------------------------------------------------------
#ifndef App_ExamsH
#define App_ExamsH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include "Moa_TypedLists.h"
namespace App_cards { class TFlashCard; } using namespace App_cards;
namespace App_cards { class TFlashCards; } using namespace App_cards;
namespace App_cards { class TFlashPage; } using namespace App_cards;
//---------------------------------------------------------------------------
namespace App_exams {
//---------------------------------------------------------------------------
class EExamError : public Exception
{
public:
	__fastcall EExamError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class TExamCard : public TObject
{
private:
	TFlashCard* FSource;
	bool FReverse;
	TFlashPage* __fastcall GetQuestion();
	TFlashPage* __fastcall GetAnswer();
	AnsiString __fastcall GetHint();
	AnsiString __fastcall GetNote();
protected:
public:
	__fastcall TExamCard(TFlashCard* ASource);
	virtual __fastcall ~TExamCard();
	virtual bool __fastcall HasHint();
	virtual bool __fastcall HasNote();
	__property TFlashCard* Source = {read=FSource};
	__property TFlashPage* Question = {read=GetQuestion};
	__property TFlashPage* Answer = {read=GetAnswer};
	__property AnsiString Hint = {read=GetHint};
	__property AnsiString Note = {read=GetNote};
	__property bool Reverse = {read=FReverse, write=FReverse};
};
//---------------------------------------------------------------------------
class TExamCards : public TObject
{
private:
	TFlashCards* FSource;
	TTypedObjectList<TExamCard>* FList;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	TExamCard* __fastcall GetItem(int AIndex);
	int __fastcall GetCount();
	__property TTypedObjectList<TExamCard>* List = {read=FList};
protected:
public:
	__fastcall TExamCards(TFlashCards* ASource);
	virtual __fastcall ~TExamCards();
	virtual void __fastcall Shuffle();
	virtual int __fastcall Find(TExamCard* AItem);
	__property TFlashCards* Source = {read=FSource};
	__property TExamCard* Items[int AIndex] = {read=GetItem};
	__property int Count = {read=GetCount};
};
//---------------------------------------------------------------------------
class TExamCardNavigator : public TObject
{
private:
	enum TState { stStarted, stEnded };
	typedef Set<TState, stStarted, stEnded> TStates;
	TTypedObjectList<TExamCard>* FList;
	TTypedObjectList<TExamCard>* FHistory;
	int FPosition;
	TStates FStates;
	TNotifyEvent FOnStart;
	TNotifyEvent FOnEnd;
	TNotifyEvent FOnProgress;
	void __fastcall Initialize(TExamCards* ACards);
	void __fastcall Uninitialize();
	TExamCard* __fastcall GetCurrentCard();
	int __fastcall GetCurrent();
	int __fastcall GetAll();
	TStates __fastcall GetStates();
	__property TTypedObjectList<TExamCard>* List = {read=FList};
	__property TTypedObjectList<TExamCard>* History = {read=FHistory};
	__property int Position = {read=FPosition};
	__property TStates States = {read=GetStates};
protected:
	virtual void __fastcall DoStart();
	virtual void __fastcall DoEnd();
	virtual void __fastcall DoProgress();
public:
	__fastcall TExamCardNavigator(TExamCards* ACards);
	virtual __fastcall ~TExamCardNavigator();
	virtual void __fastcall Start();
	virtual bool __fastcall CanStart();
	virtual bool __fastcall IsStarted();
	virtual void __fastcall End();
	virtual bool __fastcall CanEnd();
	virtual bool __fastcall IsEnded();
	virtual void __fastcall Retry(int AInterval);
	virtual bool __fastcall CanRetry();
	virtual void __fastcall GoBack();
	virtual bool __fastcall CanGoBack();
	virtual void __fastcall GoForward();
	virtual bool __fastcall CanGoForward();
	__property TExamCard* CurrentCard = {read=GetCurrentCard};
	__property int Current = {read=GetCurrent};
	__property int All = {read=GetAll};
	__property TNotifyEvent OnStart = {read=FOnStart, write=FOnStart};
	__property TNotifyEvent OnEnd = {read=FOnEnd, write=FOnEnd};
	__property TNotifyEvent OnProgress = {read=FOnProgress, write=FOnProgress};
};
//---------------------------------------------------------------------------
class TExamStepNavigator : public TObject
{
private:
	enum TStep { stQuestion, stHint, stAnswer };
	typedef Set<TStep, stQuestion, stAnswer> TSteps;
	TSteps FSteps;
	TNotifyEvent FOnShowQuestion;
	TNotifyEvent FOnShowHint;
	TNotifyEvent FOnShowAnswer;
	TSteps __fastcall GetSteps();
	__property TSteps Steps = {read=GetSteps};
protected:
	virtual void __fastcall DoShowQuestion();
	virtual void __fastcall DoShowHint();
	virtual void __fastcall DoShowAnswer();
public:
	__fastcall TExamStepNavigator();
	virtual __fastcall ~TExamStepNavigator();
	virtual void __fastcall Proceed();
	virtual bool __fastcall CanProceed();
	virtual void __fastcall ShowQuestion();
	virtual bool __fastcall CanShowQuestion();
	virtual bool __fastcall IsQuestionShown();
	virtual void __fastcall ShowHint();
	virtual bool __fastcall CanShowHint();
	virtual bool __fastcall IsHintShown();
	virtual void __fastcall ShowAnswer();
	virtual bool __fastcall CanShowAnswer();
	virtual bool __fastcall IsAnswerShown();
	virtual void __fastcall Reset();
	__property TNotifyEvent OnShowQuestion = {read=FOnShowQuestion, write=FOnShowQuestion};
	__property TNotifyEvent OnShowHint = {read=FOnShowHint, write=FOnShowHint};
	__property TNotifyEvent OnShowAnswer = {read=FOnShowAnswer, write=FOnShowAnswer};
};
//---------------------------------------------------------------------------
class TExamTimekeeper : public TObject
{
private:
	TTimer* FTimer;
	bool FEnabled;
	bool FTimeUp;
	TNotifyEvent FOnTimeUp;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	void __fastcall TimerTimer(TObject* ASender);
	int __fastcall GetTimeLimit();
	void __fastcall SetTimeLimit(int ATimeLimit);
	__property TTimer* Timer = {read=FTimer};
protected:
	virtual void __fastcall DoTimeUp();
public:
	__fastcall TExamTimekeeper();
	virtual __fastcall ~TExamTimekeeper();
	virtual void __fastcall Start();
	virtual void __fastcall Stop();
	virtual void __fastcall Reset();
	__property int TimeLimit = {read=GetTimeLimit, write=SetTimeLimit};
	__property bool TimeUp = {read=FTimeUp};
	__property TNotifyEvent OnTimeUp = {read=FOnTimeUp, write=FOnTimeUp};
};
//---------------------------------------------------------------------------
struct TExamRules
{
	bool Shuffle;
	bool Reverse;
	bool AutoHint;
	bool AutoAnswer;
	bool LimitTime;
	int TimeLimit;
	bool ChargeRetry;
	int RetryInterval;
	bool operator ==(const TExamRules& AOther) const;
	bool operator !=(const TExamRules& AOther) const;
};
//---------------------------------------------------------------------------
class TExam : public TObject
{
private:
	TExamCards* FCards;
	TExamCardNavigator* FCardNavigator;
	TExamStepNavigator* FStepNavigator;
	TExamTimekeeper* FTimekeeper;
	TExamRules FRules;
	TNotifyEvent FOnStart;
	TNotifyEvent FOnEnd;
	TNotifyEvent FOnProgress;
	TNotifyEvent FOnShowQuestion;
	TNotifyEvent FOnShowHint;
	TNotifyEvent FOnShowAnswer;
	TNotifyEvent FOnTimeUp;
	void __fastcall Initialize(TFlashCards* ACards);
	void __fastcall Uninitialize();
	void __fastcall CardNavigatorStart(TObject* ASender);
	void __fastcall CardNavigatorEnd(TObject* ASender);
	void __fastcall CardNavigatorProgress(TObject* ASender);
	void __fastcall StepNavigatorShowQuestion(TObject* ASender);
	void __fastcall StepNavigatorShowHint(TObject* ASender);
	void __fastcall StepNavigatorShowAnswer(TObject* ASender);
	void __fastcall TimekeeperTimeUp(TObject* ASender);
	TExamCard* __fastcall GetCurrentCard();
	int __fastcall GetCurrent();
	int __fastcall GetAll();
	__property TExamCards* Cards = {read=FCards};
	__property TExamCardNavigator* CardNavigator = {read=FCardNavigator};
	__property TExamStepNavigator* StepNavigator = {read=FStepNavigator};
	__property TExamTimekeeper* Timekeeper = {read=FTimekeeper};
	__property TExamRules Rules = {read=FRules};
protected:
	virtual void __fastcall DoStart();
	virtual void __fastcall DoEnd();
	virtual void __fastcall DoProgress();
	virtual void __fastcall DoShowQuestion();
	virtual void __fastcall DoShowHint();
	virtual void __fastcall DoShowAnswer();
	virtual void __fastcall DoTimeUp();
public:
	__fastcall TExam(TFlashCards* ACards, const TExamRules& ARules);
	virtual __fastcall ~TExam();
	virtual void __fastcall Start();
	virtual bool __fastcall CanStart();
	virtual bool __fastcall IsStarted();
	virtual bool __fastcall IsEnded();
	virtual void __fastcall Stop();
	virtual bool __fastcall CanStop();
	virtual void __fastcall Proceed();
	virtual bool __fastcall CanProceed();
	virtual void __fastcall Retry();
	virtual bool __fastcall CanRetry();
	virtual void __fastcall ShowHint();
	virtual bool __fastcall CanShowHint();
	virtual void __fastcall ShowAnswer();
	virtual bool __fastcall CanShowAnswer();
	virtual void __fastcall GoBack();
	virtual bool __fastcall CanGoBack();
	virtual void __fastcall GoForward();
	virtual bool __fastcall CanGoForward();
	__property TExamCard* CurrentCard  = {read=GetCurrentCard};
	__property int Current = {read=GetCurrent};
	__property int All = {read=GetAll};
	__property TNotifyEvent OnStart = {read=FOnStart, write=FOnStart};
	__property TNotifyEvent OnEnd = {read=FOnEnd, write=FOnEnd};
	__property TNotifyEvent OnProgress = {read=FOnProgress, write=FOnProgress};
	__property TNotifyEvent OnShowQuestion = {read=FOnShowQuestion, write=FOnShowQuestion};
	__property TNotifyEvent OnShowHint = {read=FOnShowHint, write=FOnShowHint};
	__property TNotifyEvent OnShowAnswer = {read=FOnShowAnswer, write=FOnShowAnswer};
	__property TNotifyEvent OnTimeUp = {read=FOnTimeUp, write=FOnTimeUp};
};
//---------------------------------------------------------------------------
} // End namespace App_exams
using namespace App_exams;
//---------------------------------------------------------------------------
#endif

