//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <Clipbrd.hpp>
#include <Dialogs.hpp>
#include <IniFiles.hpp>
#include "App_Cards.h"
#include "App_Data.h"
#include "App_ExamRulesDialog.h"
#include "App_FileHandler.h"
#include "App_FileHistory.h"
#include "App_FileHistoryDialog.h"
#include "App_Views.h"
#include "Moa_AutoPointers.h"
#include "Moa_Files.h"
#include "Moa_Sets.h"
#include "Moa_ShellLinks.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_filehandler {
//---------------------------------------------------------------------------
const AnsiString TFileHandler::ProgramTitle = "Flashexam";
const AnsiString TFileHandler::FileHistoryFile = ExtractFilePath(Application->ExeName) + "History.ini";
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Initialize()
{
	try {
		FFileHistory = new TFileHistory(FileHistoryFile);
		FFileHistory->Limit = 30;
		FExamRules.Shuffle = true;
		FExamRules.Reverse = false;
		FExamRules.AutoHint = false;
		FExamRules.AutoAnswer = false;
		FExamRules.LimitTime = false;
		FExamRules.TimeLimit = 3000;
		FExamRules.ChargeRetry = true;
		FExamRules.RetryInterval = 4;
		FOpenDialog = new TOpenDialog(NULL);
		FOpenDialog->Title = "Open";
		FOpenDialog->Filter = "Text (*.txt)|*.txt|All (*.*)|*.*";
		FOpenDialog->FilterIndex = 1;
		FOpenDialog->Options = FOpenDialog->Options
			<< ofFileMustExist;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Uninitialize()
{
	delete FOpenDialog;
	FOpenDialog = NULL;

	delete FExam;
	FExam = NULL;

	delete FDeck;
	FDeck = NULL;

	delete FFileHistory;
	FFileHistory = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::OpenFile(const AnsiString& AFile)
{
	AnsiString LFile = AFile;
	if(ExtractFileDrive(LFile).Length() == 0 && IsOpen()) {
		LFile = ExpandFileName(ExtractFilePath(File) + AFile);
	}
	SHELLEXECUTEINFO LInfo = {0};
	LInfo.cbSize = sizeof(SHELLEXECUTEINFO);
	LInfo.fMask = 0;
	LInfo.lpVerb = "open";
	LInfo.lpFile = LFile.c_str();
	LInfo.nShow = SW_SHOW;
	ShellExecuteEx(&LInfo);
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::ExamStart(TObject* ASender)
{
	MainView->Clear();
	MainView->Status = "Started";
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::ExamEnd(TObject* ASender)
{
	MainView->Clear();
	MainView->WriteLine(Exam->Current == Exam->All ? "Done." : "Stopped.");
	MainView->Status = "Ready to start an exam";
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::ExamProgress(TObject* ASender)
{
	if(Exam->Current == Exam->All) {
		MainView->Clear();
		MainView->WriteLine("No more cards.");
		MainView->Status = "Proceed to end the exam";
	}
	else {
		MainView->Clear();
		MainView->Status = AnsiString().sprintf("Card %d of %d", Exam->Current + 1, Exam->All);
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::ExamShowQuestion(TObject* ASender)
{
	MainView->WriteLine("Q. " + Exam->CurrentCard->Question->Text);
	if(Exam->CurrentCard->Question->HasFile()) {
		OpenFile(Exam->CurrentCard->Question->File);
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::ExamShowHint(TObject* ASender)
{
	MainView->WriteBlankLine();
	MainView->WriteLine("Hint: " + Exam->CurrentCard->Hint);
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::ExamShowAnswer(TObject* ASender)
{
	MainView->WriteBlankLine();
	MainView->WriteLine("A. " + Exam->CurrentCard->Answer->Text);
	if(Exam->CurrentCard->Answer->HasFile()) {
		OpenFile(Exam->CurrentCard->Answer->File);
	}
	if(Exam->CurrentCard->HasNote()) {
		MainView->WriteBlankLine();
		MainView->WriteLine("Note: " + Exam->CurrentCard->Note);
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::ExamTimeUp(TObject* ASender)
{
	MainView->WriteBlankLine();
	MainView->WriteLine("Time is up!");
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SetFile(const AnsiString& AFile)
{
	if(AFile == File) {
		return;
	}
	FFile = AFile;
	if(File.Length() > 0) {
		SetCurrentDir(ExtractFileDir(File));
		MainView->Caption = ProgramTitle + " - " + ExtractFileName(File);
		FileHistory->Reload();
		FileHistory->Add(File);
		FileHistory->UpdateFile();
	}
	else {
		MainView->Caption = ProgramTitle;
	}
}
//---------------------------------------------------------------------------
__fastcall TFileHandler::TFileHandler(TMainView* AMainView) :
	THandler(),
	FFile(),
	FFileHistory(NULL),
	FDeck(NULL),
	FExam(NULL),
	FExamRules(),
	FMainView(AMainView),
	FOpenDialog(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TFileHandler::~TFileHandler()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Open()
{
	if(!CanOpen()) {
		HandleFeatureUnavailable();
	}
	OpenDialog->FileName = AnsiString();
	if(!OpenDialog->Execute()) {
		Abort();
	}
	Open(OpenDialog->FileName);
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Open(const AnsiString& AFile)
{
	if(!CanOpen()) {
		HandleFeatureUnavailable();
	}
	AnsiString LFile = AFile; // For the case AFile refers to FFile
	if(IsOpen()) {
		Close();
	}
	TAutoPointer<TFlashDeck> LDeck = new TFlashDeck();
	LDeck->LoadFromFile(LFile);
	FDeck = LDeck.Release();
	SetFile(LFile);
	MainView->Text = AnsiString();
	MainView->Status = "Ready to start an exam";
	ShowInformation();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanOpen()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Reopen()
{
	if(!CanReopen()) {
		HandleFeatureUnavailable();
	}
	Open(File);
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanReopen()
{
	return IsOpen();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Close()
{
	if(!CanClose()) {
		HandleFeatureUnavailable();
	}
	if(IsPlaying()) {
		if(Application->MessageBox(
			"The program stops the current exam.",
			Application->Title.c_str(),
			MB_OKCANCEL | MB_ICONEXCLAMATION | MB_SETFOREGROUND) != IDOK) {
			Abort();
		}
		Stop();
	}
	delete FDeck;
	FDeck = NULL;
	SetFile(AnsiString());
	MainView->Text = AnsiString();
	MainView->Status = "No deck";
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanClose()
{
	return IsOpen();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::DropFiles(const TStringDynArray& AFiles)
{
	if(!CanDropFiles()) {
		HandleFeatureUnavailable();
	}
	if(AFiles.Length != 1) {
		throw EHandlerError("Specify one file.");
	}
	AnsiString LFile = AFiles[0];
	LFile = TShortcutFileUtilities::EnsureDereferenced(LFile);
	LFile = TFileProperties::GetLongPath(LFile);
	Open(LFile);
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanDropFiles()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::ShowFileHistory()
{
	if(!CanShowFileHistory()) {
		HandleFeatureUnavailable();
	}
	TAppFileHistoryValues LValues;
	FileHistory->Reload();
	LValues.FileHistory = FileHistory;
	LValues.Index = 0;
	TAppFileHistoryDialog::EnsureCreated();
	AppFileHistoryDialog->Values = LValues;
	if(!AppFileHistoryDialog->Execute()) {
		return;
	}
	LValues = AppFileHistoryDialog->Values;
	Open(FileHistory->Items[LValues.Index]);
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanShowFileHistory()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::CopyDeck()
{
	TAutoPointer<TFlashEntries> LEntries = new TFlashEntries();
	Deck->WriteData(LEntries);
	Clipboard()->AsText = LEntries->AsText;
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanCopyDeck()
{
	return IsOpen();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::CopyInformation()
{
	TAutoPointer<TFlashEntries> LEntries = new TFlashEntries();
	Deck->About->WriteData(LEntries);
	Clipboard()->AsText = LEntries->AsText;
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanCopyInformation()
{
	return IsOpen();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::CopyCards()
{
	TAutoPointer<TFlashEntries> LEntries = new TFlashEntries();
	Deck->Cards->WriteData(LEntries);
	Clipboard()->AsText = LEntries->AsText;
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanCopyCards()
{
	return IsOpen();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::CopyCurrentCard()
{
	TAutoPointer<TFlashEntries> LEntries = new TFlashEntries();
	Exam->CurrentCard->Source->WriteData(LEntries);
	Clipboard()->AsText = LEntries->AsText;
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanCopyCurrentCard()
{
	return IsPlaying() && Exam->Current < Exam->All;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::CopyMessages()
{
	if(!CanCopyMessages()) {
		HandleFeatureUnavailable();
	}
	Clipboard()->AsText = MainView->Text;
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanCopyMessages()
{
	return MainView->Text.Length() > 0;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::ShowInformation()
{
	if(!CanShowInformation()) {
		HandleFeatureUnavailable();
	}
	MainView->Clear();
	MainView->WriteLine(Deck->About->HasTitle() ? Deck->About->Title.c_str() : "(No title)");
	MainView->WriteBlankLine();
	MainView->WriteLine(Deck->About->HasDescription() ? Deck->About->Description.c_str() : "(No description)");
	MainView->Status = AnsiString().sprintf("%d cards", Deck->Cards->Count);
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanShowInformation()
{
	return IsOpen() && !IsPlaying();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Start()
{
	if(!CanStart()) {
		HandleFeatureUnavailable();
	}
	try {
		TAutoPointer<TExam> LExam = new TExam(Deck->Cards, ExamRules);
		LExam->OnStart = ExamStart;
		LExam->OnEnd = ExamEnd;
		LExam->OnProgress = ExamProgress;
		LExam->OnShowQuestion = ExamShowQuestion;
		LExam->OnShowHint = ExamShowHint;
		LExam->OnShowAnswer = ExamShowAnswer;
		LExam->OnTimeUp = ExamTimeUp;
		FExam = LExam.Release();
		Exam->Start();
	}
	catch(...) {
		delete FExam;
		FExam = NULL;
		throw;
	}
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanStart()
{
	return IsOpen() && !IsPlaying() && Deck->Cards->Count > 0;
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Stop()
{
	if(!CanStop()) {
		HandleFeatureUnavailable();
	}
	Exam->Stop();
	delete FExam;
	FExam = NULL;
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanStop()
{
	return IsPlaying() && Exam->CanStop();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Proceed()
{
	if(!CanProceed()) {
		HandleFeatureUnavailable();
	}
	Exam->Proceed();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanProceed()
{
	return IsPlaying() && Exam->CanProceed();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::Retry()
{
	if(!CanRetry()) {
		HandleFeatureUnavailable();
	}
	Exam->Retry();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanRetry()
{
	return IsPlaying() && Exam->CanRetry();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::ShowHint()
{
	if(!CanShowHint()) {
		HandleFeatureUnavailable();
	}
	Exam->ShowHint();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanShowHint()
{
	return IsPlaying() && Exam->CanShowHint();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::GoBack()
{
	if(!CanGoBack()) {
		HandleFeatureUnavailable();
	}
	Exam->GoBack();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanGoBack()
{
	return IsPlaying() && Exam->CanGoBack();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::GoForward()
{
	if(!CanGoForward()) {
		HandleFeatureUnavailable();
	}
	Exam->GoForward();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanGoForward()
{
	return IsPlaying() && Exam->CanGoForward();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SetExamRules(const TExamRules& AExamRules)
{
	if(!CanSetExamRules()) {
		HandleFeatureUnavailable();
	}
	if(AExamRules == ExamRules) {
		return;
	}
	FExamRules = AExamRules;
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanSetExamRules()
{
	return !IsPlaying();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::ShowExamRules()
{
	if(!CanShowExamRules()) {
		HandleFeatureUnavailable();
	}
	TAppExamRulesDialog::EnsureCreated()->Values = ExamRules;
	if(!AppExamRulesDialog->Execute()) {
		Abort();
	}
	SetExamRules(AppExamRulesDialog->Values);
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::CanShowExamRules()
{
	return !IsPlaying() && CanSetExamRules();
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::IsOpen()
{
	return Deck != NULL;
}
//---------------------------------------------------------------------------
bool __fastcall TFileHandler::IsPlaying()
{
	return Exam != NULL && Exam->IsStarted() && !Exam->IsEnded();
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::LoadSettings(TMemIniFile* AFile)
{
	AnsiString LSection = "FileHandler.ExamRules";
	{
		TExamRules LExamRules = ExamRules;
		LExamRules.Shuffle = AFile->ReadBool(
			LSection,
			"Shuffle",
			LExamRules.Shuffle);
		LExamRules.Reverse = AFile->ReadBool(
			LSection,
			"Reverse",
			LExamRules.Reverse);
		LExamRules.AutoHint = AFile->ReadBool(
			LSection,
			"AutoHint",
			LExamRules.AutoHint);
		LExamRules.AutoAnswer = AFile->ReadBool(
			LSection,
			"AutoAnswer",
			LExamRules.AutoAnswer);
		LExamRules.LimitTime = AFile->ReadBool(
			LSection,
			"LimitTime",
			LExamRules.LimitTime);
		LExamRules.TimeLimit = AFile->ReadInteger(
			LSection,
			"TimeLimit",
			LExamRules.TimeLimit);
		LExamRules.ChargeRetry = AFile->ReadBool(
			LSection,
			"ChargeRetry",
			LExamRules.ChargeRetry);
		LExamRules.RetryInterval = AFile->ReadInteger(
			LSection,
			"RetryInterval",
			LExamRules.RetryInterval);
		FExamRules = LExamRules;
	}
	LSection = "FileHandler.OpenDialog";
	{
		OpenDialog->FilterIndex = AFile->ReadInteger(
			LSection,
			"FilterIndex",
			OpenDialog->FilterIndex);
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileHandler::SaveSettings(TMemIniFile* AFile)
{
	AnsiString LSection = "FileHandler.ExamRules";
	{
		AFile->WriteBool(
			LSection,
			"Shuffle",
			ExamRules.Shuffle);
		AFile->WriteBool(
			LSection,
			"Reverse",
			ExamRules.Reverse);
		AFile->WriteBool(
			LSection,
			"AutoHint",
			ExamRules.AutoHint);
		AFile->WriteBool(
			LSection,
			"AutoAnswer",
			ExamRules.AutoAnswer);
		AFile->WriteBool(
			LSection,
			"LimitTime",
			ExamRules.LimitTime);
		AFile->WriteInteger(
			LSection,
			"TimeLimit",
			ExamRules.TimeLimit);
		AFile->WriteBool(
			LSection,
			"ChargeRetry",
			ExamRules.ChargeRetry);
		AFile->WriteInteger(
			LSection,
			"RetryInterval",
			ExamRules.RetryInterval);
	}
	LSection = "FileHandler.OpenDialog";
	{
		AFile->WriteInteger(
			LSection,
			"FilterIndex",
			OpenDialog->FilterIndex);
	}
}
//---------------------------------------------------------------------------
} // End namespace App_filehandler
//---------------------------------------------------------------------------

