//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <IniFiles.hpp>
#include <Math.hpp>
#include "App_FileHistory.h"
#include "Moa_AutoPointers.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_filehistory {
//---------------------------------------------------------------------------
void __fastcall TFileHistory::Initialize()
{
	try {
		FList = new TStringList();
		Load();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileHistory::Uninitialize()
{
	delete FList;
	FList = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TFileHistory::Load()
{
	TAutoPointer<TStringList> LList = new TStringList();
	{
		TAutoPointer<TMemIniFile> LFile = new TMemIniFile(File);
		int LCuont = LFile->ReadInteger("History", "Count", 0);
		LList->Capacity = LCuont;
		for(int LI = 0; LI < LCuont; ++LI) {
			LList->Add(LFile->ReadString(
				"History",
				AnsiString("Item") + LI,
				AnsiString()));
		}
	}
	List->Assign(LList);
}
//---------------------------------------------------------------------------
void __fastcall TFileHistory::Save()
{
	TAutoPointer<TMemIniFile> LFile = new TMemIniFile(File);
	LFile->Clear();
	LFile->WriteInteger("History", "Count", Count);
	for(int LI = 0; LI < Count; ++LI) {
		LFile->WriteString(
			"History",
			AnsiString("Item") + LI,
			Items[LI]);
	}
	LFile->UpdateFile();
}
//---------------------------------------------------------------------------
AnsiString __fastcall TFileHistory::GetItem(int AIndex)
{
	return List->Strings[AIndex];
}
//---------------------------------------------------------------------------
int __fastcall TFileHistory::GetCount()
{
	return List->Count;
}
//---------------------------------------------------------------------------
void __fastcall TFileHistory::SetCount(int ACount)
{
	if(ACount < 0 || ACount > Count) {
		throw EListError("The program cannot set the number of items.");
	}
	while(List->Count > ACount) {
		List->Delete(List->Count - 1);
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileHistory::SetLimit(int ALimit)
{
	if(ALimit < 0) {
		throw EListError("The program cannot set the limit.");
	}
	if(Count > ALimit) {
		Count = ALimit;
	}
	FLimit = ALimit;
}
//---------------------------------------------------------------------------
__fastcall TFileHistory::TFileHistory(const AnsiString& AFile) :
	TObject(),
	FList(NULL),
	FLimit(9),
	FFile(AFile)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TFileHistory::~TFileHistory()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
int __fastcall TFileHistory::Add(const AnsiString& AItem)
{
	if(Limit == 0) {
		return -1;
	}
	Remove(AItem);
	if(Count == Limit) {
		Count = Count - 1;
	}
	int LIndex = 0;
	List->Insert(LIndex, AItem);
	return LIndex;
}
//---------------------------------------------------------------------------
int __fastcall TFileHistory::Remove(const AnsiString& AItem)
{
	int LIndex = Find(AItem);
	if(LIndex < 0) {
		return -1;
	}
	Delete(LIndex);
	return LIndex;
}
//---------------------------------------------------------------------------
void __fastcall TFileHistory::Delete(int AIndex)
{
	List->Delete(AIndex);
}
//---------------------------------------------------------------------------
void __fastcall TFileHistory::Clear()
{
	List->Clear();
}
//---------------------------------------------------------------------------
int __fastcall TFileHistory::Find(const AnsiString& AItem)
{
	return List->IndexOf(AItem);
}
//---------------------------------------------------------------------------
void __fastcall TFileHistory::Reload()
{
	Load();
}
//---------------------------------------------------------------------------
void __fastcall TFileHistory::UpdateFile()
{
	Save();
}
//---------------------------------------------------------------------------
} // End namespace App_filehistory
//---------------------------------------------------------------------------

