//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <IniFiles.hpp>
#include "App_SettingsHandler.h"
#include "Moa_AutoPointers.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_settingshandler {
//---------------------------------------------------------------------------
const AnsiString TSettingsHandler::File = ExtractFilePath(Application->ExeName) + "Settings.ini";
//---------------------------------------------------------------------------
void __fastcall TSettingsHandler::Initialize()
{
	try {
		FHandlers = new TTypedObjectList<THandler>(false);
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TSettingsHandler::Uninitialize()
{
	delete FHandlers;
	FHandlers = NULL;
}
//---------------------------------------------------------------------------
__fastcall TSettingsHandler::TSettingsHandler() :
	THandler(),
	FHandlers(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TSettingsHandler::~TSettingsHandler()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TSettingsHandler::Load()
{
	if(!CanLoad()) {
		HandleFeatureUnavailable();
	}
	if(!FileExists(File)) {
		return;
	}
	TAutoPointer<TMemIniFile> LFile = new TMemIniFile(File);
	for(int LI = 0; LI < Handlers->Count; ++LI) {
		Handlers->Items[LI]->LoadSettings(LFile);
	}
}
//---------------------------------------------------------------------------
bool __fastcall TSettingsHandler::CanLoad()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TSettingsHandler::Save()
{
	if(!CanLoad()) {
		HandleFeatureUnavailable();
	}
	TAutoPointer<TMemIniFile> LFile = new TMemIniFile(File);
	LFile->Clear();
	for(int LI = 0; LI < Handlers->Count; ++LI) {
		Handlers->Items[LI]->SaveSettings(LFile);
	}
	LFile->UpdateFile();
}
//---------------------------------------------------------------------------
bool __fastcall TSettingsHandler::CanSave()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TSettingsHandler::Register(THandler* AHandler)
{
	Handlers->Add(AHandler);
}
//---------------------------------------------------------------------------
void __fastcall TSettingsHandler::LoadSettings(TMemIniFile* AFile)
{
	AnsiString LSection = "SettingsHandler";
	{
		SetCurrentDir(AFile->ReadString(
			LSection,
			"CurrentDirectory",
			GetCurrentDir()));
	}
}
//---------------------------------------------------------------------------
void __fastcall TSettingsHandler::SaveSettings(TMemIniFile* AFile)
{
	AnsiString LSection = "SettingsHandler";
	{
		AFile->WriteString(
			LSection,
			"CurrentDirectory",
			GetCurrentDir());
	}
}
//---------------------------------------------------------------------------
} // End namespace App_settingshandler
//---------------------------------------------------------------------------

