//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <IniFiles.hpp>
#include "App_ToolsDialog.h"
#include "App_ToolsHandler.h"
#include "Moa_Files.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_toolshandler {
//---------------------------------------------------------------------------
bool __fastcall TToolsOptions::operator ==(const TToolsOptions& AOther) const
{
	if(&AOther == this) {
		return true;
	}
	return
		EditorFile == AOther.EditorFile;
}
//---------------------------------------------------------------------------
bool __fastcall TToolsOptions::operator !=(const TToolsOptions& AOther) const
{
	return !(*this == AOther);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
const AnsiString TToolsHandler::TemplateFile = ExtractFilePath(Application->ExeName) + "Templates\\Default.txt";
//---------------------------------------------------------------------------
void __fastcall TToolsHandler::Initialize()
{
	try {
		FOptions.EditorFile = AnsiString();
		FTemplateSaveDialog = new TSaveDialog(NULL);
		FTemplateSaveDialog->Title = "Create Template";
		FTemplateSaveDialog->Filter = "Text (*.txt)|*.txt|All (*.*)|*.*";
		FTemplateSaveDialog->FilterIndex = 1;
		FTemplateSaveDialog->Options = FTemplateSaveDialog->Options
			<< ofOverwritePrompt;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TToolsHandler::Uninitialize()
{
	delete FTemplateSaveDialog;
	FTemplateSaveDialog = NULL;
}
//---------------------------------------------------------------------------
__fastcall TToolsHandler::TToolsHandler() :
	THandler(),
	FOptions(),
	FTemplateSaveDialog(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TToolsHandler::~TToolsHandler()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TToolsHandler::OpenWithEditor(const AnsiString& AFile)
{
	if(!CanOpenWithEditor()) {
		HandleFeatureUnavailable();
	}
	AnsiString LEditorFile = Options.EditorFile;
	AnsiString LFile;
	AnsiString LParameters;
	if(LEditorFile.Length() > 0) {
		LFile = LEditorFile;
		LParameters = AnsiString().sprintf("\"%s\"", AFile.c_str());
	}
	else {
		LFile = AnsiString().sprintf("\"%s\"", AFile.c_str());
		LParameters = AnsiString();
	}
	SHELLEXECUTEINFO LInfo = {0};
	LInfo.cbSize = sizeof(SHELLEXECUTEINFO);
	LInfo.fMask = SEE_MASK_FLAG_NO_UI;
	LInfo.lpVerb = "open";
	LInfo.lpFile = LFile.c_str();
	LInfo.lpParameters = static_cast<LPCSTR>(LParameters.data());
	LInfo.nShow = SW_SHOW;
	if(!ShellExecuteEx(&LInfo)) {
		throw EHandlerError("The program could not open the deck with an editor.");
	}
}
//---------------------------------------------------------------------------
bool __fastcall TToolsHandler::CanOpenWithEditor()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TToolsHandler::CreateTemplate()
{
	if(!CanCreateTemplate()) {
		HandleFeatureUnavailable();
	}
	try {
		TemplateSaveDialog->FileName = "Untitled.txt";
		if(!TemplateSaveDialog->Execute())  {
			Abort();
		}
		TRegularFile::Copy(TemplateFile, TemplateSaveDialog->FileName, true);
	}
	catch(EFileError& LException) {
		throw EHandlerError("The program could not create a template file.");
	}
}
//---------------------------------------------------------------------------
bool __fastcall TToolsHandler::CanCreateTemplate()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TToolsHandler::SetOptions(const TToolsOptions& AOptions)
{
	if(AOptions == Options) {
		return;
	}
	FOptions = AOptions;
}
//---------------------------------------------------------------------------
bool __fastcall TToolsHandler::CanSetOptions()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TToolsHandler::ShowOptions()
{
	if(!CanShowOptions()) {
		HandleFeatureUnavailable();
	}
	TAppToolsDialog::EnsureCreated()->Values = Options;
	if(!AppToolsDialog->Execute()) {
		Abort();
	}
	SetOptions(AppToolsDialog->Values);
}
//---------------------------------------------------------------------------
bool __fastcall TToolsHandler::CanShowOptions()
{
	return CanSetOptions();
}
//---------------------------------------------------------------------------
void __fastcall TToolsHandler::LoadSettings(TMemIniFile* AFile)
{
	AnsiString LSection = "ToolsHandler.Options";
	{
		TToolsOptions LOptions = Options;
		LOptions.EditorFile = AFile->ReadString(
			LSection,
			"EditorFile",
			LOptions.EditorFile);
		SetOptions(LOptions);
	}
}
//---------------------------------------------------------------------------
void __fastcall TToolsHandler::SaveSettings(TMemIniFile* AFile)
{
	AnsiString LSection = "ToolsHandler.Options";
	{
		TToolsOptions LOptions = Options;
		AFile->WriteString(
			LSection,
			"EditorFile",
			LOptions.EditorFile);
		SetOptions(LOptions);
	}
}
//---------------------------------------------------------------------------
} // End namespace App_toolshandler
//---------------------------------------------------------------------------

