//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <ComCtrls.hpp>
#include <IniFiles.hpp>
#include <StdCtrls.hpp>
#include "App_ViewDialog.h"
#include "App_ViewHandler.h"
#include "Moa_Forms.h"
#include "Moa_Sets.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace App_viewhandler {
//---------------------------------------------------------------------------
bool __fastcall TControlOptions::operator ==(const TControlOptions& AOther) const
{
	if(&AOther == this) {
		return true;
	}
	return
		ShowToolbar == AOther.ShowToolbar &&
		ShowStatusBar == AOther.ShowStatusBar;
}
//---------------------------------------------------------------------------
bool __fastcall TControlOptions::operator !=(const TControlOptions& AOther) const
{
	return !(*this == AOther);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
bool __fastcall TMessageOptions::operator ==(const TMessageOptions& AOther) const
{
	if(&AOther == this) {
		return true;
	}
	return
		BackgroundColor == AOther.BackgroundColor &&
		ForegroundColor == AOther.ForegroundColor &&
		FontName == AOther.FontName &&
		FontStyles == AOther.FontStyles &&
		FontSize == AOther.FontSize;
}
//---------------------------------------------------------------------------
bool __fastcall TMessageOptions::operator !=(const TMessageOptions& AOther) const
{
	return !(*this == AOther);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
bool __fastcall TViewOptions::operator ==(const TViewOptions& AOther) const
{
	if(&AOther == this) {
		return true;
	}
	return
		ControlOptions == AOther.ControlOptions &&
		MessageOptions == AOther.MessageOptions;
}
//---------------------------------------------------------------------------
bool __fastcall TViewOptions::operator !=(const TViewOptions& AOther) const
{
	return !(*this == AOther);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
TControlOptions __fastcall TViewHandler::GetControlOptions()
{
	TControlOptions LControlOptions;
	LControlOptions.ShowToolbar = Toolbar->Visible;
	LControlOptions.ShowStatusBar = StatusBar->Visible;
	return LControlOptions;
}
//---------------------------------------------------------------------------
TMessageOptions __fastcall TViewHandler::GetMessageOptions()
{
	TMessageOptions LMessageOptions;
	LMessageOptions.BackgroundColor = StaticText->Color;
	LMessageOptions.ForegroundColor = StaticText->Font->Color;
	LMessageOptions.FontName = StaticText->Font->Name;
	LMessageOptions.FontStyles = StaticText->Font->Style;
	LMessageOptions.FontSize = StaticText->Font->Size;
	return LMessageOptions;
}
//---------------------------------------------------------------------------
TViewOptions __fastcall TViewHandler::GetViewOptions()
{
	TViewOptions LViewOptions;
	LViewOptions.ControlOptions = ControlOptions;
	LViewOptions.MessageOptions = MessageOptions;
	return LViewOptions;
}
//---------------------------------------------------------------------------
__fastcall TViewHandler::TViewHandler(TForm* AForm, TToolBar* AToolbar, TStatusBar* AStatusBar, TStaticText* AStaticText) :
	THandler(),
	FForm(AForm),
	FToolbar(AToolbar),
	FStatusBar(AStatusBar),
	FStaticText(AStaticText)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TViewHandler::~TViewHandler()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TViewHandler::SetControlOptions(const TControlOptions& AControlOptions)
{
	if(!CanSetControlOptions()) {
		HandleFeatureUnavailable();
	}
	if(AControlOptions == ControlOptions) {
		return;
	}
	Toolbar->Visible = AControlOptions.ShowToolbar;
	StatusBar->Visible = AControlOptions.ShowStatusBar;
}
//---------------------------------------------------------------------------
bool __fastcall TViewHandler::CanSetControlOptions()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TViewHandler::SetMessageOptions(const TMessageOptions& AMessageOptions)
{
	if(!CanSetMessageOptions()) {
		HandleFeatureUnavailable();
	}
	if(AMessageOptions == MessageOptions) {
		return;
	}
	StaticText->Color = AMessageOptions.BackgroundColor;
	StaticText->Font->Color = AMessageOptions.ForegroundColor;
	StaticText->Font->Name = AMessageOptions.FontName;
	StaticText->Font->Style = AMessageOptions.FontStyles;
	StaticText->Font->Size = AMessageOptions.FontSize;
}
//---------------------------------------------------------------------------
bool __fastcall TViewHandler::CanSetMessageOptions()
{
	return true;
}
//---------------------------------------------------------------------------
void __fastcall TViewHandler::SetViewOptions(const TViewOptions& AViewOptions)
{
	if(!CanSetViewOptions()) {
		HandleFeatureUnavailable();
	}
	if(AViewOptions == ViewOptions) {
		return;
	}
	SetControlOptions(AViewOptions.ControlOptions);
	SetMessageOptions(AViewOptions.MessageOptions);
}
//---------------------------------------------------------------------------
bool __fastcall TViewHandler::CanSetViewOptions()
{
	return CanSetControlOptions() && CanSetMessageOptions();
}
//---------------------------------------------------------------------------
void __fastcall TViewHandler::Customize()
{
	if(!CanCustomize()) {
		HandleFeatureUnavailable();
	}
	TAppViewDialog::EnsureCreated()->Values = ViewOptions;
	if(!AppViewDialog->Execute()) {
		Abort();
	}
	SetViewOptions(AppViewDialog->Values);
}
//---------------------------------------------------------------------------
bool __fastcall TViewHandler::CanCustomize()
{
	return CanSetViewOptions();
}
//---------------------------------------------------------------------------
void __fastcall TViewHandler::LoadSettings(TMemIniFile* AFile)
{
	AnsiString LSection = "ViewHandler.Form";
	{
		TRect LBounds = TFormUtilities::GetNormalBounds(Form);
		LBounds.left = AFile->ReadInteger(
			LSection,
			"Left",
			LBounds.left);
		LBounds.top = AFile->ReadInteger(
			LSection,
			"Top",
			LBounds.top);
		LBounds.right = AFile->ReadInteger(
			LSection,
			"Right",
			LBounds.right);
		LBounds.bottom = AFile->ReadInteger(
			LSection,
			"Bottom",
			LBounds.bottom);
		TFormUtilities::SetNormalBounds(Form, LBounds);
	}
	LSection = "ViewHandler.ControlOptions";
	{
		TControlOptions LControlOptions = ControlOptions;
		LControlOptions.ShowToolbar = AFile->ReadBool(
			LSection,
			"ShowToolbar",
			LControlOptions.ShowToolbar);
		LControlOptions.ShowStatusBar = AFile->ReadBool(
			LSection,
			"ShowStatusBar",
			LControlOptions.ShowStatusBar);
		SetControlOptions(LControlOptions);
	}
	LSection = "ViewHandler.MessageOptions";
	{
		TMessageOptions LMessageOptions = MessageOptions;
		LMessageOptions.BackgroundColor = static_cast<TColor>(AFile->ReadInteger(
			LSection,
			"BackgroundColor",
			static_cast<int>(LMessageOptions.BackgroundColor)));
		LMessageOptions.ForegroundColor = static_cast<TColor>(AFile->ReadInteger(
			LSection,
			"ForegroundColor",
			static_cast<int>(StaticText->Color)));
		LMessageOptions.FontName = AFile->ReadString(
			LSection,
			"FontName",
			LMessageOptions.FontName);
		LMessageOptions.FontStyles = TSetUtilities<TFontStyle, fsBold, fsStrikeOut>::FromString(AFile->ReadString(
			LSection,
			"FontStyles",
			TSetUtilities<TFontStyle, fsBold, fsStrikeOut>::ToString(LMessageOptions.FontStyles)));
		LMessageOptions.FontSize = AFile->ReadInteger(
			LSection,
			"FontSize",
			LMessageOptions.FontSize);
		SetMessageOptions(LMessageOptions);
	}
}
//---------------------------------------------------------------------------
void __fastcall TViewHandler::SaveSettings(TMemIniFile* AFile)
{
	AnsiString LSection = "ViewHandler.Form";
	{
		TRect LBounds = TFormUtilities::GetNormalBounds(Form);
		AFile->WriteInteger(
			LSection,
			"Left",
			LBounds.left),
		AFile->WriteInteger(
			LSection,
			"Top",
			LBounds.top),
		AFile->WriteInteger(
			LSection,
			"Right",
			LBounds.right),
		AFile->WriteInteger(
			LSection,
			"Bottom",
			LBounds.bottom);
	}
	LSection = "ViewHandler.ControlOptions";
	{
		TControlOptions LControlOptions = ControlOptions;
		AFile->WriteBool(
			LSection,
			"ShowToolbar",
			LControlOptions.ShowToolbar);
		AFile->WriteBool(
			LSection,
			"ShowStatusBar",
			LControlOptions.ShowStatusBar);
	}
	LSection = "ViewHandler.MessageOptions";
	{
		TMessageOptions LMessageOptions = MessageOptions;
		AFile->WriteInteger(
			LSection,
			"BackgroundColor",
			LMessageOptions.BackgroundColor);
		AFile->WriteInteger(
			LSection,
			"ForegroundColor",
			LMessageOptions.ForegroundColor);
		AFile->WriteString(
			LSection,
			"FontName",
			LMessageOptions.FontName);
		AFile->WriteString(
			LSection,
			"FontStyles",
			TSetUtilities<TFontStyle, fsBold, fsStrikeOut>::ToString(LMessageOptions.FontStyles));
		AFile->WriteInteger(
			LSection,
			"FontSize",
			LMessageOptions.FontSize);
	}
}
//---------------------------------------------------------------------------
} // End namespace App_viewhandler
//---------------------------------------------------------------------------

