//---------------------------------------------------------------------------
// Version: 1.1.0
// Date: 2012-02-08 UTC+09:00
// Status: Beta
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Moa_AutoPointers.h"
#include "Moa_Files.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moa_files {
//---------------------------------------------------------------------------
bool __fastcall TFile::Exists(const AnsiString& APath)
{
	return ::GetFileAttributes(APath.c_str()) != 0xFFFFFFFF;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
AnsiString __fastcall TRegularFile::CreateTemporary(const AnsiString& ADirectory, const AnsiString& APrefix)
{
	TAutoArray<TCHAR> LBuffer = new TCHAR[MAX_PATH];
	::ZeroMemory(LBuffer, sizeof(TCHAR) * MAX_PATH);
	if(::GetTempFileName(ADirectory.c_str(), APrefix.c_str(), 0, LBuffer) == 0) {
		throw EFileError("The program could not create a temporary file.");
	}
	return AnsiString(LBuffer);
}
//---------------------------------------------------------------------------
void __fastcall TRegularFile::Move(const AnsiString& ASource, const AnsiString& ADestination)
{
	if(!::MoveFile(ASource.c_str(), ADestination.c_str())) {
		throw EFileError("The program could not move the file.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TRegularFile::Copy(const AnsiString& ASource, const AnsiString& ADestination, bool AOverwrite)
{
	if(!::CopyFile(ASource.c_str(), ADestination.c_str(), !AOverwrite)) {
		throw EFileError("The program could not copy the file.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TRegularFile::Delete(const AnsiString& APath)
{
	if(!::DeleteFile(APath.c_str())) {
		throw EFileError("The program could not delete the file.");
	}
}
//---------------------------------------------------------------------------
bool __fastcall TRegularFile::Exists(const AnsiString& APath)
{
	DWORD LAttributes = ::GetFileAttributes(APath.c_str());
	return LAttributes != 0xFFFFFFFF && (LAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TDirectory::Create(const AnsiString& APath)
{
	if(!::CreateDirectory(APath.c_str(), NULL)) {
		throw EFileError("The program could not create the directory.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TDirectory::CreateAll(const AnsiString& APath)
{
	if(APath.Length() == 0 || (!Exists(APath) && !Sysutils::ForceDirectories(APath))) {
		throw EFileError("The program could not create the directories.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TDirectory::Move(const AnsiString& ASource, const AnsiString& ADestination)
{
	if(!::MoveFile(ASource.c_str(), ADestination.c_str())) {
		throw EFileError("The program could not move the directory.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TDirectory::Delete(const AnsiString& APath)
{
	if(!::RemoveDirectory(APath.c_str())) {
		throw EFileError("The program could not delete the directory.");
	}
}
//---------------------------------------------------------------------------
bool __fastcall TDirectory::Exists(const AnsiString& APath)
{
	DWORD LAttributes = ::GetFileAttributes(APath.c_str());
	return LAttributes != 0xFFFFFFFF && (LAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
AnsiString __fastcall TSpecialFiles::GetCurrentDirectory()
{
	DWORD LBufferSize = ::GetCurrentDirectory(0, NULL);
	if(LBufferSize == 0) {
		throw EFileError("The program could not get the current directory.");
	}
	TAutoArray<TCHAR> LBuffer = new TCHAR[LBufferSize];
	DWORD LDataSize = ::GetCurrentDirectory(LBufferSize, LBuffer);
	if(LDataSize == 0 || LDataSize >= LBufferSize) {
		throw EFileError("The program could not get the current directory.");
	}
	return AnsiString(LBuffer, LDataSize);
}
//---------------------------------------------------------------------------
void __fastcall TSpecialFiles::SetCurrentDirectory(const AnsiString& APath)
{
	if(!::SetCurrentDirectory(APath.c_str())) {
		throw EFileError("The program could not set the current directory.");
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TSpecialFiles::GetSystemDirectory()
{
	UINT LBufferSize = ::GetSystemDirectory(NULL, 0);
	if(LBufferSize == 0) {
		throw EFileError("The program could not get the system directory.");
	}
	TAutoArray<TCHAR> LBuffer = new TCHAR[LBufferSize];
	UINT LDataSize = ::GetSystemDirectory(LBuffer, LBufferSize);
	if(LDataSize == 0 || LDataSize >= LBufferSize) {
		throw EFileError("The program could not get the system directory.");
	}
	return AnsiString(LBuffer, LDataSize);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TSpecialFiles::GetTemporaryDirectory()
{
	DWORD LBufferSize = ::GetTempPath(0, NULL);
	if(LBufferSize == 0) {
		throw EFileError("The program could not get the temporary directory.");
	}
	TAutoArray<TCHAR> LBuffer = new TCHAR[LBufferSize];
	DWORD LDataSize = ::GetTempPath(LBufferSize, LBuffer);
	if(LDataSize == 0 || LDataSize >= LBufferSize) {
		throw EFileError("The program could not get the temporary directory.");
	}
	return AnsiString(LBuffer, LDataSize);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TSpecialFiles::GetWindowsDirectory()
{
	UINT LBufferSize = ::GetWindowsDirectory(NULL, 0);
	if(LBufferSize == 0) {
		throw EFileError("The program could not get the Windows directory.");
	}
	TAutoArray<TCHAR> LBuffer = new TCHAR[LBufferSize];
	UINT LDataSize = ::GetWindowsDirectory(LBuffer, LBufferSize);
	if(LDataSize == 0 || LDataSize >= LBufferSize) {
		throw EFileError("The program could not get the Windows directory.");
	}
	return AnsiString(LBuffer, LDataSize);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
AnsiString __fastcall TFileProperties::GetShortPath(const AnsiString& APath)
{
	// TO-DO: Trap the path that contains wildcard characters

	DWORD LBufferSize = ::GetShortPathName(APath.c_str(), NULL, 0);
	if(LBufferSize == 0) {
		throw EFileError("The program could not get the short path of the file.");
	}
	TAutoArray<TCHAR> LBuffer = new TCHAR[LBufferSize];
	DWORD LDataSize = ::GetShortPathName(APath.c_str(), LBuffer, LBufferSize);
	if(LDataSize == 0 || LDataSize >= LBufferSize) {
		throw EFileError("The program could not get the short path of the file.");
	}
	return AnsiString(LBuffer, LDataSize);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TFileProperties::GetLongPath(const AnsiString& APath)
{
	// TO-DO: Trap the path that contains wildcard characters

	DWORD LBufferSize = ::GetLongPathName(APath.c_str(), NULL, 0);
	if(LBufferSize == 0) {
		throw EFileError("The program could not get the long path of the file.");
	}
	TAutoArray<TCHAR> LBuffer = new TCHAR[LBufferSize];
	DWORD LDataSize = ::GetLongPathName(APath.c_str(), LBuffer, LBufferSize);
	if(LDataSize == 0 || LDataSize >= LBufferSize) {
		throw EFileError("The program could not get the long path of the file.");
	}
	return AnsiString(LBuffer, LDataSize);
}
//---------------------------------------------------------------------------
DWORDLONG __fastcall TFileProperties::GetSize(const AnsiString& APath)
{
	WIN32_FIND_DATA LData = GetFindData(APath);
	ULARGE_INTEGER LSize = {0};
	LSize.LowPart = LData.nFileSizeLow;
	LSize.HighPart = LData.nFileSizeHigh;
	return LSize.QuadPart;
}
//---------------------------------------------------------------------------
DWORD __fastcall TFileProperties::GetAttributes(const AnsiString& APath)
{
	DWORD LAttributes = ::GetFileAttributes(APath.c_str());
	if(LAttributes == 0xFFFFFFFF) {
		throw EFileError("The program could not get the attributes of the file.");
	}
	return LAttributes;
}
//---------------------------------------------------------------------------
void __fastcall TFileProperties::SetAttributes(const AnsiString& APath, DWORD AAttributes)
{
	if(!::SetFileAttributes(APath.c_str(), AAttributes)) {
		throw EFileError("The program could not set the attributes of the file.");
	}
}
//---------------------------------------------------------------------------
FILETIME __fastcall TFileProperties::GetCreationTime(const AnsiString& APath)
{
	FILETIME LTime = {0};
	GetTimes(APath, &LTime, NULL, NULL);
	return LTime;
}
//---------------------------------------------------------------------------
void __fastcall TFileProperties::SetCreationTime(const AnsiString& APath, const FILETIME& ATime)
{
	SetTimes(APath, &ATime, NULL, NULL);
}
//---------------------------------------------------------------------------
FILETIME __fastcall TFileProperties::GetLastWriteTime(const AnsiString& APath)
{
	FILETIME LTime = {0};
	GetTimes(APath, NULL, &LTime, NULL);
	return LTime;
}
//---------------------------------------------------------------------------
void __fastcall TFileProperties::SetLastWriteTime(const AnsiString& APath, const FILETIME& ATime)
{
	SetTimes(APath, NULL, &ATime, NULL);
}
//---------------------------------------------------------------------------
FILETIME __fastcall TFileProperties::GetLastAccessTime(const AnsiString& APath)
{
	FILETIME LTime = {0};
	GetTimes(APath, NULL, NULL, &LTime);
	return LTime;
}
//---------------------------------------------------------------------------
void __fastcall TFileProperties::SetLastAccessTime(const AnsiString& APath, const FILETIME& ATime)
{
	SetTimes(APath, NULL, NULL, &ATime);
}
//---------------------------------------------------------------------------
void __fastcall TFileProperties::GetTimes(const AnsiString& APath, FILETIME* ACreationTime, FILETIME* ALastWriteTime, FILETIME* ALastAccessTime)
{
	WIN32_FIND_DATA LData = GetFindData(APath);
	if(ACreationTime != NULL) {
		*ACreationTime = LData.ftCreationTime;
	}
	if(ALastWriteTime != NULL) {
		*ALastWriteTime = LData.ftLastWriteTime;
	}
	if(ALastAccessTime != NULL) {
		*ALastAccessTime = LData.ftLastAccessTime;
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileProperties::SetTimes(const AnsiString& APath, const FILETIME* ACreationTime, const FILETIME* ALastWriteTime, const FILETIME* ALastAccessTime)
{
	HANDLE LHandle = ::CreateFile(
		APath.c_str(),
		GENERIC_WRITE,
		0,
		NULL,
		OPEN_EXISTING,
		(TFileProperties::GetAttributes(APath) & FILE_ATTRIBUTE_DIRECTORY) != 0 ?
			FILE_ATTRIBUTE_NORMAL | FILE_FLAG_BACKUP_SEMANTICS :
			FILE_ATTRIBUTE_NORMAL,
		NULL);
	if(LHandle == INVALID_HANDLE_VALUE) {
		throw EFileError("The program could not set the times of the file.");
	}
	try {
		if(!::SetFileTime(LHandle, ACreationTime, ALastAccessTime, ALastWriteTime)) {
			throw EFileError("The program could not set the times of the file.");
		}
	}
	__finally {
		::CloseHandle(LHandle);
	}
}
//---------------------------------------------------------------------------
WIN32_FIND_DATA __fastcall TFileProperties::GetFindData(const AnsiString& APath)
{
	WIN32_FIND_DATA LData = {0};
	{
		HANDLE LHandle = ::FindFirstFile(APath.c_str(), &LData);
		if(LHandle == INVALID_HANDLE_VALUE) {
			throw EFileError("The program could not find the file.");
		}
		::FindClose(LHandle);
	}
	return LData;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
bool __fastcall TPath::IsAbsolute(const AnsiString& APath)
{
	return Sysutils::ExtractFileDrive(APath).Length() > 0;
}
//---------------------------------------------------------------------------
bool __fastcall TPath::IsRelative(const AnsiString& APath)
{
	return !IsAbsolute(APath);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
bool __fastcall TFileEnumerator::FindFirst()
{
	::ZeroMemory(&FData, sizeof(WIN32_FIND_DATA));
	FHandle = ::FindFirstFile(Path.c_str(), &FData);
	return Handle != INVALID_HANDLE_VALUE;
}
//---------------------------------------------------------------------------
bool __fastcall TFileEnumerator::FindNext()
{
	::ZeroMemory(&FData, sizeof(WIN32_FIND_DATA));
	return ::FindNextFile(Handle, &FData);
}
//---------------------------------------------------------------------------
void __fastcall TFileEnumerator::Finish()
{
	if(Handle == INVALID_HANDLE_VALUE) {
		return;
	}
	if(Handle != NULL) {
		::FindClose(FHandle);
	}
	FHandle = INVALID_HANDLE_VALUE;
}
//---------------------------------------------------------------------------
WIN32_FIND_DATA __fastcall TFileEnumerator::GetCurrent()
{
	return FData;
}
//---------------------------------------------------------------------------
__fastcall TFileEnumerator::TFileEnumerator(const AnsiString& APath) :
	TObject(),
	FPath(APath),
	FHandle(NULL),
	FData()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TFileEnumerator::~TFileEnumerator()
{
	Finish();
}
//---------------------------------------------------------------------------
bool __fastcall TFileEnumerator::MoveNext()
{
	if(Handle == NULL) {
		return FindFirst();
	}
	else {
		if(FindNext()) {
			return true;
		}
		else {
			Finish();
			return false;
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileEnumerator::Reset()
{
	Finish();
	FHandle = NULL;
}
//---------------------------------------------------------------------------
} // End namespace Moa_files
//---------------------------------------------------------------------------

