//---------------------------------------------------------------------------
// Version: 1.1.0
// Date: 2012-02-08 UTC+09:00
// Status: Beta
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Moa_AutoPointers.h"
#include "Moa_ShellLinks.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moa_shelllinks {
//---------------------------------------------------------------------------
const int TShellLink::MaxCharacterLength = 2;
	// In a multi-byte character set, the maximum length of a character is two bytes.
//---------------------------------------------------------------------------
void __fastcall TShellLink::Initialize()
{
	try {
		::CoInitialize(NULL);
		if(FAILED(CoCreateInstance(
			CLSID_ShellLink,
			NULL,
			CLSCTX_INPROC_SERVER,
			IID_IShellLink,
			reinterpret_cast<void**>(&FShellLink))) ||
			FAILED(ShellLink->QueryInterface(
			IID_IPersistFile,
			reinterpret_cast<void**>(&FPersistFile)))) {
			throw EShellLinkError("The program could not create a shell link.");
		}
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::Uninitialize()
{
	if(FPersistFile != NULL) {
		FPersistFile->Release();
		FPersistFile = NULL;
	}
	if(FShellLink != NULL) {
		FShellLink->Release();
		FShellLink = NULL;
	}
	::CoUninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::HandlePropertyGetError()
{
	throw EShellLinkError("The program could not get a property of a shell link.");
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::HandlePropertySetError()
{
	throw EShellLinkError("The program could not set a property of a shell link.");
}
//---------------------------------------------------------------------------
AnsiString __fastcall TShellLink::GetTarget()
{
	int LBufferSize = MAX_PATH;
	TAutoArray<CHAR> LBuffer = new CHAR[LBufferSize];
	WIN32_FIND_DATA LFindData = {0};
	if(FAILED(ShellLink->GetPath(LBuffer, LBufferSize, &LFindData, SLGP_UNCPRIORITY))) {
		HandlePropertyGetError();
	}
	return LBuffer.Pointer;
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::SetTarget(const AnsiString& ATarget)
{
	if(FAILED(ShellLink->SetPath(ATarget.c_str()))) {
		HandlePropertySetError();
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TShellLink::GetWorkingDirectory()
{
	int LBufferSize = MAX_PATH;
	TAutoArray<CHAR> LBuffer = new CHAR[LBufferSize];
	if(FAILED(ShellLink->GetWorkingDirectory(LBuffer, LBufferSize))) {
		HandlePropertyGetError();
	}
	return LBuffer.Pointer;
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::SetWorkingDirectory(const AnsiString& AWorkingDirectory)
{
	if(FAILED(ShellLink->SetWorkingDirectory(AWorkingDirectory.c_str()))) {
		HandlePropertySetError();
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TShellLink::GetArguments()
{
	int LBufferSize = MAX_PATH + MaxCharacterLength;
	while(true) {
		TAutoArray<CHAR> LBuffer = new CHAR[LBufferSize];
		if(FAILED(ShellLink->GetArguments(LBuffer, LBufferSize))) {
			HandlePropertyGetError();
		}
		if(LBufferSize - ::lstrlen(LBuffer) - 1 < MaxCharacterLength) {
			LBuffer = NULL;
			LBufferSize *= 2;
			continue;
		}
		return LBuffer.Pointer;
	}
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::SetArguments(const AnsiString& AArguments)
{
	if(FAILED(ShellLink->SetArguments(AArguments.c_str()))) {
		HandlePropertySetError();
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TShellLink::GetDescription()
{
	int LBufferSize = MAX_PATH + MaxCharacterLength;
	while(true) {
		TAutoArray<CHAR> LBuffer = new CHAR[LBufferSize];
		if(FAILED(ShellLink->GetDescription(LBuffer, LBufferSize))) {
			HandlePropertyGetError();
		}
		if(LBufferSize - ::lstrlen(LBuffer) - 1 < MaxCharacterLength) {
			LBuffer = NULL;
			LBufferSize *= 2;
			continue;
		}
		return LBuffer.Pointer;
	}
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::SetDescription(const AnsiString& ADescription)
{
	if(FAILED(ShellLink->SetDescription(ADescription.c_str()))) {
		HandlePropertySetError();
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TShellLink::GetIconFile()
{
	AnsiString LPath;
	int LIndex;
	GetIconLocation(LPath, LIndex);
	return LPath;
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::SetIconFile(const AnsiString& AIconFile)
{
	SetIconLocation(AIconFile, IconIndex);
}
//---------------------------------------------------------------------------
int __fastcall TShellLink::GetIconIndex()
{
	AnsiString LIconFile;
	int LIconIndex;
	GetIconLocation(LIconFile, LIconIndex);
	return LIconIndex;
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::SetIconIndex(int AIconIndex)
{
	SetIconLocation(IconFile, AIconIndex);
}
//---------------------------------------------------------------------------
int __fastcall TShellLink::GetShowCommand()
{
	int LShowCommand;
	if(FAILED(ShellLink->GetShowCmd(&LShowCommand))) {
		HandlePropertyGetError();
	}
	return LShowCommand;
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::SetShowCommand(int AShowCommand)
{
	if(FAILED(ShellLink->SetShowCmd(AShowCommand))) {
		HandlePropertySetError();
	}
}
//---------------------------------------------------------------------------
WORD __fastcall TShellLink::GetHotKey()
{
	WORD LHotKey;
	if(FAILED(ShellLink->GetHotkey(&LHotKey))) {
		HandlePropertyGetError();
	}
	return LHotKey;
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::SetHotKey(WORD AHotKey)
{
	if(FAILED(ShellLink->SetHotkey(AHotKey))) {
		HandlePropertySetError();
	}
}
//---------------------------------------------------------------------------
__fastcall TShellLink::TShellLink() :
	TObject(),
	FShellLink(NULL),
	FPersistFile(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TShellLink::~TShellLink()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::Resolve(HWND AParentWindow, DWORD AFlags)
{
	if(FAILED(ShellLink->Resolve(AParentWindow, AFlags))) {
		throw EShellLinkError("The program could not resolve a shell link.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::LoadFromFile(const AnsiString& APath)
{
	if(FAILED(PersistFile->Load(WideString(APath).c_bstr(), STGM_READ | STGM_SHARE_DENY_WRITE))) {
		throw EShellLinkError("The program could not load a shell link.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::SaveToFile(const AnsiString& APath)
{
	if(FAILED(PersistFile->Save(WideString(APath).c_bstr(), true))) {
		throw EShellLinkError("The program could not save a shell link.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::GetIconLocation(AnsiString& APath, int& AIndex)
{
	int LBufferSize = MAX_PATH;
	TAutoArray<CHAR> LBuffer = new CHAR[LBufferSize];
	if(FAILED(ShellLink->GetIconLocation(LBuffer, LBufferSize, &AIndex))) {
		HandlePropertyGetError();
	}
	APath = LBuffer.Pointer;
}
//---------------------------------------------------------------------------
void __fastcall TShellLink::SetIconLocation(const AnsiString& APath, int AIndex)
{
	if(FAILED(ShellLink->SetIconLocation(APath.c_str(), AIndex))) {
		HandlePropertySetError();
	}
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
AnsiString __fastcall TShortcutFileUtilities::EnsureDereferenced(const AnsiString& APath)
{
	if(!IsShortcutFile(APath)) {
		return APath;
	}
	return Dereference(APath);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TShortcutFileUtilities::Dereference(const AnsiString& APath)
{
	TAutoPointer<TShellLink> LShellLink = new TShellLink();
	LShellLink->LoadFromFile(APath);
	return LShellLink->Target;
}
//---------------------------------------------------------------------------
bool __fastcall TShortcutFileUtilities::IsShortcutFile(const AnsiString& APath)
{
	return ::lstrcmpi(ExtractFileExt(ExpandFileName(APath)).c_str(), ".lnk") == 0;
}
//---------------------------------------------------------------------------
} // End namespace Moa_shelllinks
//---------------------------------------------------------------------------

